import json

from pygeai.assistant.clients import AssistantClient
from pygeai.cli.commands import Command, Option, ArgumentsEnum
from pygeai.cli.commands.builders import build_help_text
from pygeai.cli.commands.common import get_llm_settings, get_welcome_data, get_messages, get_welcome_data_feature_list, \
    get_welcome_data_example_prompt
from pygeai.cli.texts.help import ASSISTANT_HELP_TEXT
from pygeai.core.common.exceptions import MissingRequirementException, WrongArgumentError
from pygeai.core.utils.console import Console


def show_help():
    """
    Displays help text in stdout
    """
    help_text = build_help_text(assistant_commands, ASSISTANT_HELP_TEXT)
    Console.write_stdout(help_text)


def get_assistant_detail(option_list: list):
    opts = {opt.name: arg for opt, arg in option_list}
    detail = opts.get("detail", "summary")
    assistant_id = opts.get("assistant_id")

    if not assistant_id:
        raise MissingRequirementException("Cannot retrieve assistant detail without assistant_id")

    client = AssistantClient()
    result = client.get_assistant_data(assistant_id, detail)
    Console.write_stdout(f"Assistant detail: \n{result}")


assistant_detail_options = [
    Option(
        "detail",
        ["--detail", "-d"],
        "Defines the level of detail required. The available options are summary (default) or full.",
        True
    ),
    Option(
        "assistant_id",
        ["--assistant-id", "--id"],
        "Assistant ID.",
        True
    ),
]


def create_assistant(option_list: list):
    opts = {opt.name: arg for opt, arg in option_list}
    assistant_type = opts.get("type", "text")
    name = opts.get("name")
    description = opts.get("description")
    prompt = opts.get("prompt")
    provider_name = opts.get("provider_name")
    model_name = opts.get("model_name")
    max_tokens = opts.get("max_tokens")
    welcome_data_title = opts.get("welcome_data_title")
    welcome_data_description = opts.get("welcome_data_description")
    
    temperature = None
    temp_arg = opts.get("temperature")
    if temp_arg:
        try:
            temperature = float(temp_arg)
        except Exception:
            raise WrongArgumentError("When defined, temperature must be a decimal numer. Example: 0.5")
    
    feature_list = []
    feature_arg = opts.get("welcome_data_feature")
    if feature_arg:
        feature_list = get_welcome_data_feature_list([], feature_arg)
    
    examples_prompt_list = []
    examples_arg = opts.get("welcome_data_example_prompt")
    if examples_arg:
        examples_prompt_list = get_welcome_data_example_prompt([], examples_arg)

    if not (assistant_type and name and prompt):
        raise MissingRequirementException("Cannot create new assistant without 'type', 'name' and 'prompt'")

    llm_settings = {}
    if provider_name or model_name or temperature or max_tokens:
        llm_settings = get_llm_settings(provider_name, model_name, temperature, max_tokens)

    welcome_data = {}
    if welcome_data_title or welcome_data_description:
        welcome_data = get_welcome_data(
            welcome_data_title,
            welcome_data_description,
            feature_list,
            examples_prompt_list
        )

    client = AssistantClient()
    result = client.create_assistant(
        assistant_type=assistant_type,
        name=name,
        prompt=prompt,
        description=description,
        llm_settings=llm_settings,
        welcome_data=welcome_data
    )
    Console.write_stdout(f"New assistant detail: \n{result}")


create_assistant_options = [
    Option(
        "type",
        ["--type", "-t"],
        'string: Type of assistant. Possible values: text, chat. (Required)',
        True
    ),
    Option(
        "name",
        ["--name", "-n"],
        'string: Name of the assistant (Required)',
        True
    ),
    Option(
        "description",
        ["--description", "-d"],
        'string: Description of the assistant.',
        True
    ),
    Option(
        "prompt",
        ["--prompt"],
        'string: Prompt for the assistant  (Required)',
        True
    ),
    Option(
        "provider_name",
        ["--provider-name", "--provider", "-p"],
        'string: provider to be used',
        True
    ),
    Option(
        "model_name",
        ["--model-name", "-m"],
        'string: name of model according to selected provider',
        True
    ),
    Option(
        "temperature",
        ["--temperature"],
        'decimal: Volatility of the assistant',
        True
    ),
    Option(
        "max_tokens",
        ["--max-tokens"],
        'integer: Max number of tokens',
        True
    ),
    Option(
        "welcome_data_title",
        ["--wd-title"],
        'Title for welcome data',
        True
    ),
    Option(
        "welcome_data_description",
        ["--wd-description"],
        'Description for welcome data',
        True
    ),
    Option(
        "welcome_data_feature",
        ["--wd-feature"],
        'Feature to include in welcome data. Must be in JSON format. It can be passed multiple times with one dictionary'
        'each time or one time with a list of dictionaries. Each dictionary must have exactly two keys: "title" and '
        '"description". Example: \'{"title": "title of feature", "description": "Description of feature"}\'',
        True
    ),
    Option(
        "welcome_data_example_prompt",
        ["--wd-example-prompt"],
        'Example prompt to include in welcome data.  Must be in JSON format. It can be passed multiple times with one dictionary'
        'each time or one time with a list of dictionaries. Each dictionary must have exactly two keys: "title", "description" '
        ' and "prompt_text". Example: \'{"title": "Title of prompt", "description": "Description of prompt", "prompt_text": "Prompt text"}\'',
        True
    ),
]


def update_assistant(option_list: list):
    opts = {opt.name: arg for opt, arg in option_list}
    assistant_id = opts.get("assistant_id")
    name = opts.get("name")
    description = opts.get("description")
    status = opts.get("status", 1)
    action = opts.get("action", "saveNewRevision")
    revision_id = opts.get("revision_id")
    prompt = opts.get("prompt")
    provider_name = opts.get("provider_name")
    model_name = opts.get("model_name")
    max_tokens = opts.get("max_tokens")
    welcome_data_title = opts.get("welcome_data_title")
    welcome_data_description = opts.get("welcome_data_description")
    
    temperature = None
    temp_arg = opts.get("temperature")
    if temp_arg:
        try:
            temperature = float(temp_arg)
        except Exception:
            raise WrongArgumentError("When defined, temperature must be a decimal numer. Example: 0.5")
    
    feature_list = []
    feature_arg = opts.get("welcome_data_feature")
    if feature_arg:
        feature_list = get_welcome_data_feature_list([], feature_arg)
    
    examples_prompt_list = []
    examples_arg = opts.get("welcome_data_example_prompt")
    if examples_arg:
        examples_prompt_list = get_welcome_data_example_prompt([], examples_arg)

    if not assistant_id:
        raise MissingRequirementException("Cannot update existing assistant without 'assistant_id'")

    if action == "save" and not revision_id:
        raise MissingRequirementException("A revision_id is necessary when updating an existing version.")

    if ((action == "saveNewRevision" or action == "savePublishNewRevision") or revision_id) and not prompt:
        raise MissingRequirementException("Prompt must be defined if revisionId is specified or in case of actions saveNewRevision and savePublishNewRevision.")

    llm_settings = {}
    if provider_name or model_name or temperature or max_tokens:
        llm_settings = get_llm_settings(provider_name, model_name, temperature, max_tokens)

    welcome_data = {}
    if welcome_data_title or welcome_data_description:
        welcome_data = get_welcome_data(welcome_data_title, welcome_data_description, feature_list, examples_prompt_list)

    client = AssistantClient()
    result = client.update_assistant(
        assistant_id=assistant_id,
        status=status,
        action=action,
        revision_id=revision_id,
        name=name,
        prompt=prompt,
        description=description,
        llm_settings=llm_settings,
        welcome_data=welcome_data
    )
    Console.write_stdout(f"Updated assistant detail: \n{result}")


update_assistant_options = [
    Option(
        "assistant_id",
        ["--assistant-id", "--id"],
        "Assistant ID.",
        True
    ),
    Option(
        "status",
        ["--status"],
        "integer: Possible values: 1:Enabled, 2:Disabled (Optional)",
        True
    ),
    Option(
        "action",
        ["--action"],
        "string: Possible values: save, saveNewRevision (default), savePublishNewRevision",
        True
    ),
    Option(
        "revision_id",
        ["--revision-id"],
        "integer: Required if user needs to update an existent revision when action = save",
        True
    ),
    Option(
        "name",
        ["--name", "-n"],
        'string: Name of the assistant (Required)',
        True
    ),
    Option(
        "description",
        ["--description", "-d"],
        'string: Description of the assistant.',
        True
    ),
    Option(
        "prompt",
        ["--prompt"],
        'string: Prompt for the assistant  (Required)',
        True
    ),
    Option(
        "provider_name",
        ["--provider-name", "--provider", "-p"],
        'string: provider to be used',
        True
    ),
    Option(
        "model_name",
        ["--model-name", "-m"],
        'string: name of model according to selected provider',
        True
    ),
    Option(
        "temperature",
        ["--temperature"],
        'decimal: Volatility of the assistant',
        True
    ),
    Option(
        "max_tokens",
        ["--max-tokens"],
        'integer: Max number of tokens',
        True
    ),
    Option(
        "welcome_data_title",
        ["--wd-title"],
        'Title for welcome data',
        True
    ),
    Option(
        "welcome_data_description",
        ["--wd-description"],
        'Description for welcome data',
        True
    ),
    Option(
        "welcome_data_feature",
        ["--wd-feature"],
        'Feature to include in welcome data. Must be in JSON format. It can be passed multiple times with one dictionary'
        'each time or one time with a list of dictionaries. Each dictionary must have exactly two keys: "title" and '
        '"description". Example: \'{"title": "title of feature", "description": "Description of feature"}\'',
        True
    ),
    Option(
        "welcome_data_example_prompt",
        ["--wd-example-prompt"],
        'Example prompt to include in welcome data.  Must be in JSON format. It can be passed multiple times with one dictionary'
        'each time or one time with a list of dictionaries. Each dictionary must have exactly two keys: "title", "description" '
        ' and "prompt_text". Example: \'{"title": "Title of prompt", "description": "Description of prompt", "prompt_text": "Prompt text"}\'',
        True
    ),
]


def delete_assistant(option_list: list):
    opts = {opt.name: arg for opt, arg in option_list}
    assistant_id = opts.get("assistant_id")

    if not assistant_id:
        raise MissingRequirementException("Cannot delete assistant without 'assistant_id'")

    client = AssistantClient()
    result = client.delete_assistant(
        assistant_id=assistant_id,
    )
    Console.write_stdout(f"Deleted assistant: \n{result}")


delete_assistant_options = [
    Option(
        "assistant_id",
        ["--assistant-id", "--id"],
        "Assistant ID.",
        True
    ),
]


def send_chat_request(option_list: list):
    opts = {opt.name: arg for opt, arg in option_list}
    assistant_name = opts.get("assistant_name")
    revision = opts.get("revision")
    revision_name = opts.get("revision_name")
    
    message_list = []
    messages_arg = opts.get("messages")
    if messages_arg:
        try:
            message_json = json.loads(messages_arg)
            if isinstance(message_json, list):
                message_list = message_json
            elif isinstance(message_json, dict):
                message_list.append(message_json)
        except Exception:
            raise WrongArgumentError(
                "Each message must be in json format: '{\"role\": \"user\", \"content\": \"message content\"}' "
                "It can be a dictionary or a list of dictionaries. Each dictionary must contain role and content")

    if not assistant_name:
        raise MissingRequirementException("Cannot send chat request without specifying assistant name")

    messages = get_messages(message_list)

    client = AssistantClient()
    result = client.send_chat_request(
        assistant_name=assistant_name,
        messages=messages,
        revision=revision,
        revision_name=revision_name
    )
    Console.write_stdout(f"Chat request response: \n{result}")


send_chat_request_options = [
    Option(
        "assistant_name",
        ["--name", "-n"],
        "string: Name of the assistant.",
        True
    ),
    Option(
        "messages",
        ["--messages", "--msg"],
        "array: Chat request data. It can be passed multiple times with single dictionary each time, or a single time "
        "as a list of dictionaries. Each dictionary instance must contain 'role' and 'content'",
        True
    ),
    Option(
        "revision",
        ["--revision"],
        "integer: Revision number.",
        True
    ),
    Option(
        "revision_name",
        ["--revision-name"],
        "string:\tName of the revision.",
        True
    ),
    Option(
        "variables",
        ["--variables", "--var"],
        "collection: A list of key/value properties (optional)",
        True
    ),
]


def get_request_status(option_list: list):
    opts = {opt.name: arg for opt, arg in option_list}
    request_id = opts.get("request_id")

    if not request_id:
        raise MissingRequirementException("Cannot retrieve status of request without request_id.")

    client = AssistantClient()
    result = client.get_request_status(request_id)
    Console.write_stdout(f"Request status: \n{result}")


request_status_options = [
    Option(
        "request_id",
        ["--request-id", "--id"],
        "Request ID.",
        True
    ),
]


def cancel_request(option_list: list):
    opts = {opt.name: arg for opt, arg in option_list}
    request_id = opts.get("request_id")

    if not request_id:
        raise MissingRequirementException("Cannot cancel request without request_id.")

    client = AssistantClient()
    result = client.cancel_request(request_id)
    Console.write_stdout(f"Cancel request detail: \n{result}")


cancel_request_options = [
    Option(
        "request_id",
        ["--request-id", "--id"],
        "Request ID.",
        True
    ),
]


assistant_commands = [
    Command(
        "help",
        ["help", "h"],
        "Display help text",
        show_help,
        ArgumentsEnum.NOT_AVAILABLE,
        [],
        []
    ),
    Command(
        "assistant_detail",
        ["get-assistant"],
        "Get assistant detail",
        get_assistant_detail,
        ArgumentsEnum.REQUIRED,
        [],
        assistant_detail_options
    ),
    Command(
        "create_assistant",
        ["create-assistant"],
        "Create new assistant",
        create_assistant,
        ArgumentsEnum.REQUIRED,
        [],
        create_assistant_options
    ),
    Command(
        "update_assistant",
        ["update-assistant"],
        "Update existing assistant",
        update_assistant,
        ArgumentsEnum.REQUIRED,
        [],
        update_assistant_options
    ),
    Command(
        "delete_assistant",
        ["delete-assistant"],
        "Delete existing assistant",
        delete_assistant,
        ArgumentsEnum.REQUIRED,
        [],
        delete_assistant_options
    ),
    Command(
        "assistant_chat",
        ["chat"],
        "Sends a chat request to the Globant Enterprise AI Assistant.",
        send_chat_request,
        ArgumentsEnum.REQUIRED,
        [],
        send_chat_request_options
    ),
    Command(
        "request_status",
        ["request-status"],
        "Retrieves the status of a request.",
        get_request_status,
        ArgumentsEnum.REQUIRED,
        [],
        request_status_options
    ),
    Command(
        "cancel_request",
        ["cancel-request"],
        "Cancels a request.",
        cancel_request,
        ArgumentsEnum.REQUIRED,
        [],
        cancel_request_options
    ),
]



from pygeai import logger
from pygeai.core.common.exceptions import APIError
from pygeai.core.embeddings.clients import EmbeddingsClient
from pygeai.core.embeddings.mappers import EmbeddingsResponseMapper
from pygeai.core.embeddings.models import EmbeddingConfiguration
from pygeai.core.embeddings.responses import EmbeddingResponse
from pygeai.core.handlers import ErrorHandler


class EmbeddingsManager:

    def __init__(self, api_key: str = None, base_url: str = None, alias: str = None):
        self.__client = EmbeddingsClient(api_key, base_url, alias)

    def generate_embeddings(
            self,
            configuration: EmbeddingConfiguration
    ) -> EmbeddingResponse:
        """
        Generates an embedding vector representing the provided input(s) using the specified model.

        This method calls the API to create a vector representation of the input(s), which can be used
        in machine learning models and algorithms. The request is sent to the embeddings API endpoint.

        :param configuration: EmbeddingConfiguration - An instance containing the configuration for generating
                                                      embeddings, including:
            - inputs: list - A list of strings representing the input(s) to embed. Each input must not
                             exceed the maximum input tokens for the model and cannot be an empty string.
            - model: str - The provider/modelId to use for generating embeddings.
            - encoding_format: str, optional - The format for the returned embeddings, either 'float'
                                                  (default) or 'base64'. Only supported by OpenAI.
            - dimensions: int, optional - The number of dimensions for the resulting output embeddings.
                                           Only supported in text-embedding-3* and later models.
            - user: str, optional - A unique identifier representing the end-user. Specific to OpenAI.
            - input_type: str, optional - Defines how the input data will be used when generating
                                           embeddings. Check if the selected embeddings model supports this option.
            - timeout: int, optional - The maximum time, in seconds, to wait for the API to respond.
                                       Defaults to 600 seconds.
            - cache: bool, optional - Whether to enable caching for the embeddings. Defaults to False.

        :return: EmbeddingResponse - A response object containing the generated embeddings and usage information.
        :raises APIError - If the API returns errors.
        """
        response_data = self.__client.generate_embeddings(
            input_list=configuration.inputs,
            model=configuration.model,
            encoding_format=configuration.encoding_format,
            dimensions=configuration.dimensions,
            user=configuration.user,
            input_type=configuration.input_type,
            timeout=configuration.timeout,
            cache=configuration.cache,
        )

        if ErrorHandler.has_errors(response_data):
            error = ErrorHandler.extract_error(response_data)
            logger.error(f"Error received while generating embeddings: {error}")
            raise APIError(f"Error received while generating embeddings: {error}")

        result = EmbeddingsResponseMapper.map_to_embedding_response(response_data)
        return result



import logging  # noqa: I001
from typing import Any

from a2a.client.auth.credentials import CredentialService
from a2a.client.middleware import ClientCallContext, ClientCallInterceptor
from a2a.types import (
    AgentCard,
    APIKeySecurityScheme,
    HTTPAuthSecurityScheme,
    In,
    OAuth2SecurityScheme,
    OpenIdConnectSecurityScheme,
)

logger = logging.getLogger(__name__)


class AuthInterceptor(ClientCallInterceptor):
    """An interceptor that automatically adds authentication details to requests.

    Based on the agent's security schemes.
    """

    def __init__(self, credential_service: CredentialService):
        self._credential_service = credential_service

    async def intercept(
        self,
        method_name: str,
        request_payload: dict[str, Any],
        http_kwargs: dict[str, Any],
        agent_card: AgentCard | None,
        context: ClientCallContext | None,
    ) -> tuple[dict[str, Any], dict[str, Any]]:
        """Applies authentication headers to the request if credentials are available."""
        if (
            agent_card is None
            or agent_card.security is None
            or agent_card.security_schemes is None
        ):
            return request_payload, http_kwargs

        for requirement in agent_card.security:
            for scheme_name in requirement:
                credential = await self._credential_service.get_credentials(
                    scheme_name, context
                )
                if credential and scheme_name in agent_card.security_schemes:
                    scheme_def_union = agent_card.security_schemes.get(
                        scheme_name
                    )
                    if not scheme_def_union:
                        continue
                    scheme_def = scheme_def_union.root

                    headers = http_kwargs.get('headers', {})

                    match scheme_def:
                        # Case 1a: HTTP Bearer scheme with an if guard
                        case HTTPAuthSecurityScheme() if (
                            scheme_def.scheme.lower() == 'bearer'
                        ):
                            headers['Authorization'] = f'Bearer {credential}'
                            logger.debug(
                                "Added Bearer token for scheme '%s' (type: %s).",
                                scheme_name,
                                scheme_def.type,
                            )
                            http_kwargs['headers'] = headers
                            return request_payload, http_kwargs

                        # Case 1b: OAuth2 and OIDC schemes, which are implicitly Bearer
                        case (
                            OAuth2SecurityScheme()
                            | OpenIdConnectSecurityScheme()
                        ):
                            headers['Authorization'] = f'Bearer {credential}'
                            logger.debug(
                                "Added Bearer token for scheme '%s' (type: %s).",
                                scheme_name,
                                scheme_def.type,
                            )
                            http_kwargs['headers'] = headers
                            return request_payload, http_kwargs

                        # Case 2: API Key in Header
                        case APIKeySecurityScheme(in_=In.header):
                            headers[scheme_def.name] = credential
                            logger.debug(
                                "Added API Key Header for scheme '%s'.",
                                scheme_name,
                            )
                            http_kwargs['headers'] = headers
                            return request_payload, http_kwargs

                # Note: Other cases like API keys in query/cookie are not handled and will be skipped.

        return request_payload, http_kwargs



from pygeai import logger
from pygeai.cli.commands import Command, Option, ArgumentsEnum
from pygeai.cli.commands.builders import build_help_text
from pygeai.cli.texts.help import SPEC_HELP_TEXT
from pygeai.core.common.exceptions import MissingRequirementException

from pygeai.core.utils.console import Console
from pygeai.lab.managers import AILabManager
from pygeai.lab.spec.loader import JSONLoader
from pygeai.lab.spec.parsers import AgentParser, ToolParser, TaskParser, AgenticProcessParser


def show_help():
    """
    Displays help text in stdout
    """
    help_text = build_help_text(spec_commands, SPEC_HELP_TEXT)
    Console.write_stdout(help_text)


def load_agent(option_list: list):
    opts = {opt.name: arg for opt, arg in option_list}
    file = opts.get("file")
    automatic_publish = opts.get("automatic_publish", False)

    if not file:
        raise MissingRequirementException("Cannot load agent definition without specifying path to JSON file.")

    agent_data = JSONLoader.load_data(file_path=file)
    if isinstance(agent_data, dict):
        agent = AgentParser.get_agent(agent_data)
        create_agent(agent, automatic_publish)
    elif isinstance(agent_data, list):
        for agent_spec in agent_data:
            agent = AgentParser.get_agent(agent_spec)
            create_agent(agent, automatic_publish)


def create_agent(agent, automatic_publish):
    try:
        created_agent = AILabManager().create_agent(
            agent=agent,
            automatic_publish=automatic_publish
        )

        Console.write_stdout(f"Created agent detail: \n{created_agent}")
    except Exception as e:
        logger.error(f"Error creating agent: {e}\nAgent data: {agent}")
        Console.write_stderr(f"Error creating agent: \n{agent}")


load_agent_options = [
    Option(
        "file",
        ["--file", "-f"],
        "Path to the file containing agent definition in JSON format.",
        True
    ),
    Option(
        "automatic_publish",
        ["--automatic-publish", "--ap"],
        "Define if reasoning strategy must be published besides being created. 0: Create as draft. 1: Create and publish.",
        True
    ),
]


def load_tool(option_list: list):
    opts = {opt.name: arg for opt, arg in option_list}
    file = opts.get("file")
    automatic_publish = opts.get("automatic_publish", False)

    if not file:
        raise MissingRequirementException("Cannot load tool definition without specifying path to JSON file.")

    tool_data = JSONLoader.load_data(file_path=file)
    if isinstance(tool_data, dict):
        tool = ToolParser.get_tool(tool_data)
        create_tool(tool, automatic_publish)
    elif isinstance(tool_data, list):
        for tool_spec in tool_data:
            tool = ToolParser.get_tool(tool_spec)
            create_tool(tool, automatic_publish)


def create_tool(tool, automatic_publish):
    try:
        created_tool = AILabManager().create_tool(
            tool=tool,
            automatic_publish=automatic_publish
        )
        Console.write_stdout(f"Created tool detail: \n{created_tool}")
    except Exception as e:
        logger.error(f"Error creating tool: {e}\nTool data: {tool}")
        Console.write_stderr(f"Error creating tool: \n{tool}")


load_tool_options = [
    Option(
        "file",
        ["--file", "-f"],
        "Path to the file containing tool definition in JSON format.",
        True
    ),
    Option(
        "automatic_publish",
        ["--automatic-publish", "--ap"],
        "Define if tool must be published besides being created. 0: Create as draft. 1: Create and publish.",
        True
    ),
]


def load_task(option_list: list):
    opts = {opt.name: arg for opt, arg in option_list}
    file = opts.get("file")
    automatic_publish = opts.get("automatic_publish", False)

    if not file:
        raise MissingRequirementException("Cannot load task definition without specifying path to JSON file.")

    task_data = JSONLoader.load_data(file_path=file)
    if isinstance(task_data, dict):
        task = TaskParser.get_task(task_data)
        create_task(task, automatic_publish)
    elif isinstance(task_data, list):
        for task_spec in task_data:
            task = TaskParser.get_task(task_spec)
            create_task(task, automatic_publish)


def create_task(task, automatic_publish):
    try:
        created_task = AILabManager().create_task(
            task=task,
            automatic_publish=automatic_publish
        )
        Console.write_stdout(f"Created task detail: \n{created_task}")
    except Exception as e:
        logger.error(f"Error creating task: {e}\nTask data: {task}")
        Console.write_stderr(f"Error creating task: \n{task}")


load_task_options = [
    Option(
        "file",
        ["--file", "-f"],
        "Path to the file containing task definition in JSON format.",
        True
    ),
    Option(
        "automatic_publish",
        ["--automatic-publish", "--ap"],
        "Define if task must be published besides being created. 0: Create as draft. 1: Create and publish.",
        True
    ),
]


def load_agentic_process(option_list: list):
    opts = {opt.name: arg for opt, arg in option_list}
    file = opts.get("file")
    automatic_publish = opts.get("automatic_publish", False)


    if not file:
        raise MissingRequirementException("Cannot load agentic process definition without specifying path to JSON file.")

    process_data = JSONLoader.load_data(file_path=file)
    if isinstance(process_data, dict):
        process = AgenticProcessParser.get_agentic_process(process_data)
        create_agentic_process(process, automatic_publish)
    elif isinstance(process_data, list):
        for process_spec in process_data:
            process = AgenticProcessParser.get_agentic_process(process_spec)
            create_agentic_process(process, automatic_publish)


def create_agentic_process(process, automatic_publish):
    try:
        created_process = AILabManager().create_process(
            process=process,
            automatic_publish=automatic_publish
        )
        Console.write_stdout(f"Created agentic process detail: \n{created_process}")
    except Exception as e:
        logger.error(f"Error creating agentic process: {e}\nProcess data: {process}")
        Console.write_stderr(f"Error creating agentic process: \n{process}")


load_agentic_process_options = [
    Option(
        "file",
        ["--file", "-f"],
        "Path to the file containing agentic process definition in JSON format.",
        True
    ),
    Option(
        "automatic_publish",
        ["--automatic-publish", "--ap"],
        "Define if agentic process must be published besides being created. 0: Create as draft. 1: Create and publish.",
        True
    ),
]


spec_commands = [
    Command(
        "help",
        ["help", "h"],
        "Display help text",
        show_help,
        ArgumentsEnum.NOT_AVAILABLE,
        [],
        []
    ),
    Command(
        "load_agent",
        ["load-agent", "la"],
        "Load agent from JSON specification",
        load_agent,
        ArgumentsEnum.REQUIRED,
        [],
        load_agent_options
    ),
    Command(
        "load_tool",
        ["load-tool", "lt"],
        "Load tool from JSON specification",
        load_tool,
        ArgumentsEnum.REQUIRED,
        [],
        load_tool_options
    ),
    Command(
        "load_task",
        ["load-task"],
        "Load task from JSON specification",
        load_task,
        ArgumentsEnum.REQUIRED,
        [],
        load_task_options
    ),
    Command(
        "load_agentic_process",
        ["load-agentic-process", "lap"],
        "Load agentic process from JSON specification",
        load_agentic_process,
        ArgumentsEnum.REQUIRED,
        [],
        load_agentic_process_options
    ),
]



from pathlib import Path

from pygeai.assistant.clients import AssistantClient
from pygeai.assistant.data_analyst.endpoints import GET_DATA_ANALYST_STATUS_V1, EXTEND_DATA_ANALYST_DATASET_V1
from pygeai.core.utils.validators import validate_status_code
from pygeai.core.utils.parsers import parse_json_response


class DataAnalystAssistantClient(AssistantClient):

    def get_status(
            self,
            assistant_id: str
    ) -> dict:
        """
        Retrieves the current status of the dataset loading process for the Data Analyst Assistant.

        :param assistant_id: str - The ID of the Data Analyst Assistant.
        :return: dict - API response containing the status or error details.
        :raises ValueError: If assistant_id is empty or invalid.
        """
        if not assistant_id or not isinstance(assistant_id, str):
            raise ValueError("assistant_id must be a non-empty string")

        endpoint = GET_DATA_ANALYST_STATUS_V1.format(id=assistant_id)
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }

        response = self.api_service.get(
            endpoint=endpoint,
            headers=headers
        )

        validate_status_code(response)
        return parse_json_response(response, "get status for assistant ID", assistant_id=assistant_id)

    def extend_dataset(
        self,
        assistant_id: str,
        file_paths: list[str]
    ) -> dict:
        """
        Uploads one or more .csv dataset files to the Data Analyst Assistant.

        :param assistant_id: str - The ID of the Data Analyst Assistant.
        :param file_paths: list[str] - List of paths to the .csv files to be uploaded.
        :return: dict - API response indicating success or failure.
        """
        endpoint = EXTEND_DATA_ANALYST_DATASET_V1.format(id=assistant_id)

        files = []
        for file_path in file_paths:
            path = Path(file_path)
            if not path.is_file():
                raise FileNotFoundError(f"File not found: {file_path}")
            if path.suffix.lower() != '.csv':
                raise ValueError(f"File must be a .csv file: {file_path}")
            files.append(("file", path.open("rb")))

        try:
            response = self.api_service.post_files_multipart(
                endpoint=endpoint,
                files=files
            )
            validate_status_code(response)
            return parse_json_response(response, "extend dataset for assistant ID", assistant_id=assistant_id)
        finally:
            for _, file_handle in files:
                file_handle.close()





from unittest import TestCase
import unittest
import uuid
from pygeai.lab.managers import AILabManager
from pygeai.lab.models import (
    AgenticProcess, KnowledgeBase, AgenticActivity, ArtifactSignal, Task, 
    UserSignal, Event, SequenceFlow
)
from pygeai.core.common.exceptions import APIError


class TestAILabCreateProcessIntegration(TestCase): 

    def setUp(self):
        """
        Set up the test environment.
        """
        self.ai_lab_manager = AILabManager()
        self.new_process = self.__load_process()
        self.created_process: AgenticProcess = None


    def tearDown(self):
        """
        Clean up after each test if necessary.
        This can be used to delete the created process
        """
        if isinstance(self.created_process, AgenticProcess):
            self.ai_lab_manager.delete_process(self.created_process.id)


    def __load_process(self):
        """
        Helper to load a complete process configuration for testing.
        """
        unique_key = str(uuid.uuid4())
        process = AgenticProcess(
            key="product_def",
            name=f"Test Process {unique_key[:8]}",
            description="This is a sample process",
            kb=KnowledgeBase(name="basic-sample", artifact_type_name=["sample-artifact"]),
            agentic_activities=[
                AgenticActivity(key="activityOne", name="First Step", task_name="basic-task", agent_name="GoogleSummarizer2", agent_revision_id=0)
            ],
            artifact_signals=[
                ArtifactSignal(key="artifact.upload.1", name="artifact.upload", handling_type="C", artifact_type_name=["sample-artifact"])
            ],
            user_signals=[
                UserSignal(key="signal_done", name="process-completed")
            ],
            start_event=Event(key="artifact.upload.1", name="artifact.upload"),
            end_event=Event(key="end", name="Done"),
            sequence_flows=[
                SequenceFlow(key="step1", source_key="artifact.upload.1", target_key="activityOne"),
                SequenceFlow(key="step2", source_key="activityOne", target_key="signal_done"),
                SequenceFlow(key="stepEnd", source_key="signal_done", target_key="end")
            ]
        )
        return process
    

    def __create_process(self, process=None, automatic_publish=False):
        """
        Helper to create a process using ai_lab_manager.
        """
        return self.ai_lab_manager.create_process(
            process=self.new_process if process is None else process,
            automatic_publish=automatic_publish
        )
    

    def test_create_process_full_data(self):
        """
        Test creating a process with all available fields populated.
        """
        self.created_process = self.__create_process()
        created_process = self.created_process
        process = self.new_process

        self.assertTrue(isinstance(created_process, AgenticProcess), "Expected a created process")

        # Assert the main fields of the created process
        self.assertIsNotNone(created_process.id)
        #self.assertEqual(created_process.key, process.key)
        self.assertEqual(created_process.name, process.name)
        self.assertEqual(created_process.description, process.description)
        self.assertEqual(created_process.status, "active")

        # Assert knowledge base
        self.assertIsNotNone(created_process.kb)
        self.assertEqual(created_process.kb.name, process.kb.name)

        # Assert agentic activities
        created_activity = created_process.agentic_activities[0]
        original_activity = process.agentic_activities[0]
        self.assertTrue(isinstance(created_activity, AgenticActivity)) 
        self.assertEqual(created_activity.key.lower(), original_activity.key.lower())
        self.assertEqual(created_activity.name, original_activity.name)
            
        # Assert artifact signals
        created_signal = created_process.artifact_signals[0]
        original_signal = process.artifact_signals[0]
        self.assertTrue(isinstance(created_signal, ArtifactSignal))        
        self.assertEqual(created_signal.key.lower(), original_signal.key.lower())
        self.assertEqual(created_signal.name, original_signal.name)
        self.assertEqual(created_signal.handling_type, original_signal.handling_type)

        # Assert user signals
        created_user_signal = created_process.user_signals[0]
        self.assertTrue(isinstance(created_user_signal, UserSignal))

        # Assert events
        self.assertIsNotNone(created_process.start_event)
        self.assertEqual(created_process.start_event.key.lower(), process.start_event.key.lower())
        self.assertEqual(created_process.start_event.name, process.start_event.name)
       
        self.assertIsNotNone(created_process.end_event)
        self.assertEqual(created_process.end_event.key.lower(), process.end_event.key.lower())
        self.assertEqual(created_process.end_event.name, process.end_event.name)

        # Assert sequence flows
        created_flow = created_process.sequence_flows[0]
        original_flow = process.sequence_flows[0]
        self.assertIsNotNone(created_process.sequence_flows)
        self.assertEqual(len(created_process.sequence_flows), len(process.sequence_flows))
            
        self.assertEqual(created_flow.key.lower(), original_flow.key.lower())
        self.assertEqual(created_flow.source_key.lower(), original_flow.source_key.lower())
        self.assertEqual(created_flow.target_key.lower(), original_flow.target_key.lower())


    def test_create_process_minimum_required_data(self):
        """
        Test creating a process with only minimum required fields (key and name).
        """
        unique_key = str(uuid.uuid4())
        self.new_process = AgenticProcess(
            key=unique_key,
            name=f"Minimal Process {unique_key[:8]}"
        )
        self.created_process = self.__create_process()
        process = self.new_process

        self.assertTrue(isinstance(self.created_process, AgenticProcess), "Expected a created process")
        self.assertIsNotNone(self.created_process.id)
        self.assertEqual(self.created_process.name, process.name)


    def test_create_process_without_key(self):
        """
        Test creating a process without a key should still work (key is optional).
        """
        self.new_process = AgenticProcess(
            name=f"Process Without Key {str(uuid.uuid4())[:8]}"
        )
        self.created_process = self.__create_process()

        self.assertTrue(isinstance(self.created_process, AgenticProcess), "Expected a created process")
        self.assertIsNotNone(self.created_process.id)
        self.assertEqual(self.created_process.name, self.new_process.name)

    
    def test_create_process_no_name(self):
        """
        Test creating a process without a name should raise an error.
        """
        test_params = [True, False]

        for auto_publish in test_params:
            with self.subTest(input=auto_publish):
                self.new_process.name = ""
                with self.assertRaises(APIError) as exception:
                    self.__create_process(automatic_publish=auto_publish)

                self.assertIn(
                    "Process name cannot be empty",
                    str(exception.exception),
                    f"Expected an error about empty process name with autopublish {'enabled' if auto_publish else 'disabled'}"
                )

   
    def test_create_process_duplicated_name(self):
        """
        Test creating a process with a duplicate name should raise an error.
        """
        test_params = [True, False]
        created_process = self.__create_process()

        for auto_publish in test_params:
            with self.subTest(input=auto_publish):
                loaded_process = self.__load_process()
                loaded_process.name = created_process.name
                with self.assertRaises(APIError) as exception:
                    self.__create_process(process = loaded_process, automatic_publish=auto_publish)

                self.assertIn(
                    f"A process with this name already exists [name={created_process.name}].",
                    str(exception.exception),
                    f"Expected an error about duplicated process name with autopublish {'enabled' if auto_publish else 'disabled'}"
                )

    
    def test_create_process_invalid_name_characters(self):
        """
        Test creating a process with invalid characters in name should raise a validation error.
        """
        test_params = [True, False]

        for auto_publish in test_params:
            with self.subTest(input=auto_publish):
                invalid_names = [
                    f"{str(uuid.uuid4())[:8]}:invalid",
                    f"{str(uuid.uuid4())[:8]}/invalid"
                ]

                for invalid_name in invalid_names:
                    new_process = self.__load_process()
                    new_process.name = invalid_name
                    
                    with self.assertRaises(APIError) as exception:
                        self.__create_process(process=new_process, automatic_publish=auto_publish)
                    
                    self.assertIn(
                        "Invalid character in name",
                        str(exception.exception),
                        f"Expected an error about invalid character in process name with autopublish {'enabled' if auto_publish else 'disabled'}"
                    )

    
    @unittest.skip("Skipping test for now.KB is required but is marked as optional")
    def test_create_process_with_no_kb(self):
        """
        Test creating a process with knowledge base in JSON format.
        """
        unique_key = str(uuid.uuid4())
        self.new_process.kb = None
        self.created_process = self.__create_process()

        self.assertIsNotNone(self.created_process.kb)
        self.assertEqual(self.created_process.kb.name, "basic-sample")


    def test_create_process_with_empty_agentic_activities_array(self):
        """
        Test creating a process with empty agentic activities array (to clear all activities).
        """
        self.new_process.agentic_activities = []
        self.new_process.sequence_flows = [] # Clear sequence flows as well because they depend on activities
        self.created_process = self.__create_process()

        # Empty array should be handled gracefully
        self.assertTrue(
            self.created_process.agentic_activities is None or 
            len(self.created_process.agentic_activities) == 0
        )


    def test_create_process_with_multiple_artifact_signals(self):
        """
        Test creating a process with multiple artifact signals.
        """
        self.new_process.artifact_signals=[
            ArtifactSignal(key="artifact.upload.1", name="artifact.upload", handling_type="C", artifact_type_name=["sample-artifact"]),
            ArtifactSignal(key="artifact.upload.2", name="artifact.upload.second", handling_type="C", artifact_type_name=["sample-artifact"])
        ]
        self.created_process = self.__create_process()

        self.assertIsNotNone(self.created_process.artifact_signals)
        self.assertEqual(len(self.created_process.artifact_signals), 2)

        first_signal = self.created_process.artifact_signals[0]
        second_signal = self.created_process.artifact_signals[1]
        
        self.assertEqual(first_signal.key.lower(), "artifact.upload.1")
        self.assertEqual(second_signal.key.lower(), "artifact.upload.2")


    def test_create_process_with_multiple_user_signals(self):
        """
        Test creating a process with multiple user signals.
        """
        self.new_process.user_signals=[
            UserSignal(key="signal_done", name="process-completed"),
            UserSignal(key="signal_cancel", name="process-cancelled")
        ]
        self.created_process = self.__create_process()

        self.assertIsNotNone(self.created_process.user_signals)
        self.assertEqual(len(self.created_process.user_signals)-1, 2)
        keys = [signal.key for signal in self.created_process.user_signals]

        assert "SIGNAL_CANCEL" in keys
        assert "SIGNAL_DONE" in keys


    def test_create_process_autopublish_disabled(self):
        """
        Test creating a process without automatic publish (default behavior).
        """
        self.created_process = self.__create_process(automatic_publish=False)
        
        # Process should be created as draft by default
        self.assertTrue(
            self.created_process.is_draft is True or 
            self.created_process.is_draft is None,  # API might not return this field for drafts
            "Expected the process to be created as draft when autopublish is disabled"
        )

   
    def test_create_process_autopublish_without_task(self):
        """
        Test creating a process with automatic publish enabled.
        """
        
        with self.assertRaises(APIError) as exception:
            self.__create_process(automatic_publish=True)

        self.assertIn(
            "Failure on process publication",
            str(exception.exception),
            "Expected an error about failure on process publication"
        )

        self.assertIn(
            "Task is required",
            str(exception.exception),
            "Expected an error about task being required"
        )


    def test_create_process_autopublish_enabled(self):
        """
        Test creating a process with automatic publish enabled.
        """
        unique_key = str(uuid.uuid4())
        task = Task(name=unique_key, description="Basic task for process", title_template="Basic Task")
        self.ai_lab_manager.create_task(task=task, automatic_publish=True)
        self.new_process.agentic_activities[0].task_name = unique_key
        
       
        self.created_process = self.__create_process(automatic_publish=True)
        self.assertTrue(
            self.created_process.is_draft is False,
            "Expected the process to be created no draft when autopublish is enabled"
        )


"""Request handler components for the A2A server."""

import logging

from a2a.server.request_handlers.default_request_handler import (
    DefaultRequestHandler,
)
from a2a.server.request_handlers.jsonrpc_handler import JSONRPCHandler
from a2a.server.request_handlers.request_handler import RequestHandler
from a2a.server.request_handlers.response_helpers import (
    build_error_response,
    prepare_response_object,
)
from a2a.server.request_handlers.rest_handler import RESTHandler


logger = logging.getLogger(__name__)

try:
    from a2a.server.request_handlers.grpc_handler import (
        GrpcHandler,  # type: ignore
    )
except ImportError as e:
    _original_error = e
    logger.debug(
        'GrpcHandler not loaded. This is expected if gRPC dependencies are not installed. Error: %s',
        _original_error,
    )

    class GrpcHandler:  # type: ignore
        """Placeholder for GrpcHandler when dependencies are not installed."""

        def __init__(self, *args, **kwargs):
            raise ImportError(
                'To use GrpcHandler, its dependencies must be installed. '
                'You can install them with \'pip install "a2a-sdk[grpc]"\''
            ) from _original_error


__all__ = [
    'DefaultRequestHandler',
    'GrpcHandler',
    'JSONRPCHandler',
    'RESTHandler',
    'RequestHandler',
    'build_error_response',
    'prepare_response_object',
]



"""Helper functions for the A2A client."""

from uuid import uuid4

from a2a.types import Message, Part, Role, TextPart


def create_text_message_object(
    role: Role = Role.user, content: str = ''
) -> Message:
    """Create a Message object containing a single TextPart.

    Args:
        role: The role of the message sender (user or agent). Defaults to Role.user.
        content: The text content of the message. Defaults to an empty string.

    Returns:
        A `Message` object with a new UUID message_id.
    """
    return Message(
        role=role, parts=[Part(TextPart(text=content))], message_id=str(uuid4())
    )



import unittest
from unittest.mock import patch, Mock, MagicMock
import json
import tempfile
import os

from pygeai.migration.comparison import (
    ResourceDiff,
    ProjectDiff,
    ProjectComparator,
    DiffFormatter
)


class TestResourceDiff(unittest.TestCase):
    def test_create_resource_diff(self):
        diff = ResourceDiff(
            resource_type="agents",
            only_in_source=[{"id": "1", "name": "Agent 1"}],
            only_in_destination=[{"id": "2", "name": "Agent 2"}],
            in_both=[{"id": "3", "name": "Agent 3"}],
            total_source=2,
            total_destination=2
        )
        
        self.assertEqual(diff.resource_type, "agents")
        self.assertEqual(len(diff.only_in_source), 1)
        self.assertEqual(len(diff.only_in_destination), 1)
        self.assertEqual(len(diff.in_both), 1)
        self.assertEqual(diff.total_source, 2)
        self.assertEqual(diff.total_destination, 2)

    def test_resource_diff_counts(self):
        diff = ResourceDiff(
            resource_type="tools",
            only_in_source=[{"id": "1"}, {"id": "2"}],
            only_in_destination=[],
            in_both=[{"id": "3"}],
            total_source=3,
            total_destination=1
        )
        
        self.assertEqual(len(diff.only_in_source), 2)
        self.assertEqual(len(diff.only_in_destination), 0)
        self.assertEqual(len(diff.in_both), 1)


class TestProjectDiff(unittest.TestCase):
    def test_create_project_diff(self):
        diff = ProjectDiff(
            source_project_id="proj-1",
            destination_project_id="proj-2",
            source_instance="https://api.test.ai",
            destination_instance="https://api.prod.ai",
            timestamp="2026-02-02T00:00:00"
        )
        
        self.assertEqual(diff.source_project_id, "proj-1")
        self.assertEqual(diff.destination_project_id, "proj-2")
        self.assertEqual(diff.source_instance, "https://api.test.ai")
        self.assertEqual(diff.destination_instance, "https://api.prod.ai")

    def test_project_diff_summary(self):
        diff = ProjectDiff(
            source_project_id="proj-1",
            destination_project_id="proj-2",
            source_instance="https://api.test.ai",
            destination_instance="https://api.prod.ai",
            summary={"resource_types_compared": 5, "total_differences": 10}
        )
        
        self.assertEqual(diff.summary["resource_types_compared"], 5)
        self.assertEqual(diff.summary["total_differences"], 10)


class TestProjectComparator(unittest.TestCase):
    @patch('pygeai.migration.comparison.AILabManager')
    def test_compare_agents_only_in_source(self, mock_lab_manager):
        mock_source_manager = Mock()
        mock_dest_manager = Mock()
        
        mock_source_agent = Mock()
        mock_source_agent.id = "agent-1"
        mock_source_agent.name = "Source Agent"
        
        mock_source_list = Mock()
        mock_source_list.agents = [mock_source_agent]
        mock_source_manager.list_agents.return_value = mock_source_list
        
        mock_dest_list = Mock()
        mock_dest_list.agents = []
        mock_dest_manager.list_agents.return_value = mock_dest_list
        
        mock_lab_manager.side_effect = [mock_source_manager, mock_dest_manager]
        
        comparator = ProjectComparator(
            from_api_key="src_key",
            from_instance="https://api.test.ai",
            from_project_id="proj-1",
            from_organization_id="org-1",
            to_api_key="dest_key",
            to_instance="https://api.prod.ai",
            to_project_id="proj-2",
            to_organization_id="org-2",
            resource_types=["agents"]
        )
        
        diff = comparator.compare()
        
        self.assertIn("agents", diff.resource_diffs)
        agent_diff = diff.resource_diffs["agents"]
        self.assertEqual(len(agent_diff.only_in_source), 1)
        self.assertEqual(len(agent_diff.only_in_destination), 0)
        self.assertEqual(agent_diff.only_in_source[0]["id"], "agent-1")

    @patch('pygeai.migration.comparison.AILabManager')
    def test_compare_agents_only_in_destination(self, mock_lab_manager):
        mock_source_manager = Mock()
        mock_dest_manager = Mock()
        
        mock_source_list = Mock()
        mock_source_list.agents = []
        mock_source_manager.list_agents.return_value = mock_source_list
        
        mock_dest_agent = Mock()
        mock_dest_agent.id = "agent-2"
        mock_dest_agent.name = "Dest Agent"
        
        mock_dest_list = Mock()
        mock_dest_list.agents = [mock_dest_agent]
        mock_dest_manager.list_agents.return_value = mock_dest_list
        
        mock_lab_manager.side_effect = [mock_source_manager, mock_dest_manager]
        
        comparator = ProjectComparator(
            from_api_key="src_key",
            from_instance="https://api.test.ai",
            from_project_id="proj-1",
            from_organization_id="org-1",
            to_api_key="dest_key",
            to_instance="https://api.prod.ai",
            to_project_id="proj-2",
            to_organization_id="org-2",
            resource_types=["agents"]
        )
        
        diff = comparator.compare()
        
        agent_diff = diff.resource_diffs["agents"]
        self.assertEqual(len(agent_diff.only_in_source), 0)
        self.assertEqual(len(agent_diff.only_in_destination), 1)
        self.assertEqual(agent_diff.only_in_destination[0]["id"], "agent-2")

    @patch('pygeai.migration.comparison.AILabManager')
    def test_compare_agents_in_both(self, mock_lab_manager):
        mock_source_manager = Mock()
        mock_dest_manager = Mock()
        
        mock_agent = Mock()
        mock_agent.id = "agent-shared"
        mock_agent.name = "Shared Agent"
        
        mock_source_list = Mock()
        mock_source_list.agents = [mock_agent]
        mock_source_manager.list_agents.return_value = mock_source_list
        
        mock_dest_list = Mock()
        mock_dest_list.agents = [mock_agent]
        mock_dest_manager.list_agents.return_value = mock_dest_list
        
        mock_lab_manager.side_effect = [mock_source_manager, mock_dest_manager]
        
        comparator = ProjectComparator(
            from_api_key="src_key",
            from_instance="https://api.test.ai",
            from_project_id="proj-1",
            from_organization_id="org-1",
            to_api_key="dest_key",
            to_instance="https://api.prod.ai",
            to_project_id="proj-2",
            to_organization_id="org-2",
            resource_types=["agents"]
        )
        
        diff = comparator.compare()
        
        agent_diff = diff.resource_diffs["agents"]
        self.assertEqual(len(agent_diff.only_in_source), 0)
        self.assertEqual(len(agent_diff.only_in_destination), 0)
        self.assertEqual(len(agent_diff.in_both), 1)
        self.assertEqual(agent_diff.in_both[0]["id"], "agent-shared")

    @patch('pygeai.migration.comparison.AILabManager')
    def test_compare_tools(self, mock_lab_manager):
        mock_source_manager = Mock()
        mock_dest_manager = Mock()
        
        mock_source_tool = Mock()
        mock_source_tool.id = "tool-1"
        mock_source_tool.name = "Tool 1"
        
        mock_source_list = Mock()
        mock_source_list.tools = [mock_source_tool]
        mock_source_manager.list_tools.return_value = mock_source_list
        
        mock_dest_list = Mock()
        mock_dest_list.tools = []
        mock_dest_manager.list_tools.return_value = mock_dest_list
        
        mock_lab_manager.side_effect = [mock_source_manager, mock_dest_manager]
        
        comparator = ProjectComparator(
            from_api_key="src_key",
            from_instance="https://api.test.ai",
            from_project_id="proj-1",
            from_organization_id="org-1",
            to_api_key="dest_key",
            to_instance="https://api.prod.ai",
            to_project_id="proj-2",
            to_organization_id="org-2",
            resource_types=["tools"]
        )
        
        diff = comparator.compare()
        
        self.assertIn("tools", diff.resource_diffs)
        tool_diff = diff.resource_diffs["tools"]
        self.assertEqual(len(tool_diff.only_in_source), 1)

    @patch('pygeai.migration.comparison.AILabManager')
    def test_compare_processes(self, mock_lab_manager):
        mock_source_manager = Mock()
        mock_dest_manager = Mock()
        
        mock_source_process = Mock()
        mock_source_process.id = "process-1"
        mock_source_process.name = "Process 1"
        
        mock_source_list = Mock()
        mock_source_list.processes = [mock_source_process]
        mock_source_manager.list_processes.return_value = mock_source_list
        
        mock_dest_list = Mock()
        mock_dest_list.processes = []
        mock_dest_manager.list_processes.return_value = mock_dest_list
        
        mock_lab_manager.side_effect = [mock_source_manager, mock_dest_manager]
        
        comparator = ProjectComparator(
            from_api_key="src_key",
            from_instance="https://api.test.ai",
            from_project_id="proj-1",
            from_organization_id="org-1",
            to_api_key="dest_key",
            to_instance="https://api.prod.ai",
            to_project_id="proj-2",
            to_organization_id="org-2",
            resource_types=["processes"]
        )
        
        diff = comparator.compare()
        
        self.assertIn("processes", diff.resource_diffs)
        process_diff = diff.resource_diffs["processes"]
        self.assertEqual(len(process_diff.only_in_source), 1)

    @patch('pygeai.migration.comparison.AILabManager')
    def test_compare_tasks(self, mock_lab_manager):
        mock_source_manager = Mock()
        mock_dest_manager = Mock()
        
        mock_source_task = Mock()
        mock_source_task.id = "task-1"
        mock_source_task.name = "Task 1"
        
        mock_source_list = Mock()
        mock_source_list.tasks = [mock_source_task]
        mock_source_manager.list_tasks.return_value = mock_source_list
        
        mock_dest_list = Mock()
        mock_dest_list.tasks = []
        mock_dest_manager.list_tasks.return_value = mock_dest_list
        
        mock_lab_manager.side_effect = [mock_source_manager, mock_dest_manager]
        
        comparator = ProjectComparator(
            from_api_key="src_key",
            from_instance="https://api.test.ai",
            from_project_id="proj-1",
            from_organization_id="org-1",
            to_api_key="dest_key",
            to_instance="https://api.prod.ai",
            to_project_id="proj-2",
            to_organization_id="org-2",
            resource_types=["tasks"]
        )
        
        diff = comparator.compare()
        
        self.assertIn("tasks", diff.resource_diffs)
        task_diff = diff.resource_diffs["tasks"]
        self.assertEqual(len(task_diff.only_in_source), 1)

    @patch('pygeai.migration.comparison.AILabManager')
    def test_compare_artifact_types(self, mock_lab_manager):
        mock_source_manager = Mock()
        mock_dest_manager = Mock()
        
        mock_source_artifact = Mock()
        mock_source_artifact.id = "artifact-1"
        mock_source_artifact.name = "Artifact 1"
        
        mock_source_list = Mock()
        mock_source_list.artifact_types = [mock_source_artifact]
        mock_source_manager.list_artifact_types.return_value = mock_source_list
        
        mock_dest_list = Mock()
        mock_dest_list.artifact_types = []
        mock_dest_manager.list_artifact_types.return_value = mock_dest_list
        
        mock_lab_manager.side_effect = [mock_source_manager, mock_dest_manager]
        
        comparator = ProjectComparator(
            from_api_key="src_key",
            from_instance="https://api.test.ai",
            from_project_id="proj-1",
            from_organization_id="org-1",
            to_api_key="dest_key",
            to_instance="https://api.prod.ai",
            to_project_id="proj-2",
            to_organization_id="org-2",
            resource_types=["artifact_types"]
        )
        
        diff = comparator.compare()
        
        self.assertIn("artifact_types", diff.resource_diffs)
        artifact_diff = diff.resource_diffs["artifact_types"]
        self.assertEqual(len(artifact_diff.only_in_source), 1)

    @patch('pygeai.migration.comparison.RAGAssistantMapper')
    @patch('pygeai.migration.comparison.RAGAssistantClient')
    @patch('pygeai.migration.comparison.AILabManager')
    def test_compare_rag_assistants(self, mock_lab_manager, mock_rag_client, mock_mapper):
        mock_source_manager = Mock()
        mock_dest_manager = Mock()
        mock_lab_manager.side_effect = [mock_source_manager, mock_dest_manager]
        
        mock_source_rag = Mock()
        mock_dest_rag = Mock()
        
        mock_assistant = Mock()
        mock_assistant.name = "Assistant 1"
        mock_mapper.map_to_rag_assistant.return_value = mock_assistant
        
        mock_source_rag.get_assistants_from_project.return_value = {
            "assistants": [{"name": "Assistant 1"}]
        }
        mock_dest_rag.get_assistants_from_project.return_value = {
            "assistants": []
        }
        
        mock_rag_client.side_effect = [mock_source_rag, mock_dest_rag]
        
        comparator = ProjectComparator(
            from_api_key="src_key",
            from_instance="https://api.test.ai",
            from_project_id="proj-1",
            from_organization_id="org-1",
            to_api_key="dest_key",
            to_instance="https://api.prod.ai",
            to_project_id="proj-2",
            to_organization_id="org-2",
            resource_types=["rag_assistants"]
        )
        
        diff = comparator.compare()
        
        self.assertIn("rag_assistants", diff.resource_diffs)
        rag_diff = diff.resource_diffs["rag_assistants"]
        self.assertEqual(len(rag_diff.only_in_source), 1)

    @patch('pygeai.migration.comparison.FileManager')
    @patch('pygeai.migration.comparison.AILabManager')
    def test_compare_files_with_org_ids(self, mock_lab_manager, mock_file_manager):
        mock_source_lab = Mock()
        mock_dest_lab = Mock()
        mock_lab_manager.side_effect = [mock_source_lab, mock_dest_lab]
        
        mock_source_file_mgr = Mock()
        mock_dest_file_mgr = Mock()
        
        mock_file = Mock()
        mock_file.id = "file-1"
        mock_file.filename = "test.txt"
        
        mock_source_list = Mock()
        mock_source_list.files = [mock_file]
        mock_source_file_mgr.get_file_list.return_value = mock_source_list
        
        mock_dest_list = Mock()
        mock_dest_list.files = []
        mock_dest_file_mgr.get_file_list.return_value = mock_dest_list
        
        mock_file_manager.side_effect = [mock_source_file_mgr, mock_dest_file_mgr]
        
        comparator = ProjectComparator(
            from_api_key="src_key",
            from_instance="https://api.test.ai",
            from_project_id="proj-1",
            to_api_key="dest_key",
            to_instance="https://api.prod.ai",
            to_project_id="proj-2",
            from_organization_id="org-1",
            to_organization_id="org-2",
            resource_types=["files"]
        )
        
        diff = comparator.compare()
        
        self.assertIn("files", diff.resource_diffs)
        file_diff = diff.resource_diffs["files"]
        self.assertEqual(len(file_diff.only_in_source), 1)

    @patch('pygeai.migration.comparison.AILabManager')
    def test_compare_files_without_org_ids_skips(self, mock_lab_manager):
        mock_source_lab = Mock()
        mock_dest_lab = Mock()
        mock_lab_manager.side_effect = [mock_source_lab, mock_dest_lab]
        
        comparator = ProjectComparator(
            from_api_key="src_key",
            from_instance="https://api.test.ai",
            from_project_id="proj-1",
            from_organization_id="org-1",
            to_api_key="dest_key",
            to_instance="https://api.prod.ai",
            to_project_id="proj-2",
            to_organization_id="org-2",
            resource_types=["files"]
        )
        
        diff = comparator.compare()
        
        self.assertIn("files", diff.resource_diffs)
        file_diff = diff.resource_diffs["files"]
        self.assertEqual(file_diff.total_source, 0)
        self.assertEqual(file_diff.total_destination, 0)

    @patch('pygeai.migration.comparison.UsageLimitManager')
    @patch('pygeai.migration.comparison.AILabManager')
    def test_compare_usage_limits_with_org_keys(self, mock_lab_manager, mock_limit_manager):
        mock_source_lab = Mock()
        mock_dest_lab = Mock()
        mock_lab_manager.side_effect = [mock_source_lab, mock_dest_lab]
        
        mock_source_limit_mgr = Mock()
        mock_dest_limit_mgr = Mock()
        
        mock_limit = Mock()
        mock_limit.model = "gpt-4"
        mock_limit.limit = 1000
        
        mock_source_response = Mock()
        mock_source_response.usage_limits = [mock_limit]
        mock_source_limit_mgr.get_all_usage_limits_from_project.return_value = mock_source_response
        
        mock_dest_response = Mock()
        mock_dest_response.usage_limits = []
        mock_dest_limit_mgr.get_latest_usage_limit_from_project.return_value = mock_dest_response
        
        mock_limit_manager.side_effect = [mock_source_limit_mgr, mock_dest_limit_mgr]
        
        comparator = ProjectComparator(
            from_api_key="src_key",
            from_instance="https://api.test.ai",
            from_project_id="proj-1",
            to_api_key="dest_key",
            to_instance="https://api.prod.ai",
            to_project_id="proj-2",
            from_organization_id="org-1",
            to_organization_id="org-2",
            from_organization_api_key="src_org_key",
            to_organization_api_key="dest_org_key",
            resource_types=["usage_limits"]
        )
        
        diff = comparator.compare()
        
        self.assertIn("usage_limits", diff.resource_diffs)
        limit_diff = diff.resource_diffs["usage_limits"]
        self.assertEqual(len(limit_diff.only_in_source), 1)

    @patch('pygeai.migration.comparison.AILabManager')
    def test_compare_usage_limits_without_org_keys_skips(self, mock_lab_manager):
        mock_source_lab = Mock()
        mock_dest_lab = Mock()
        mock_lab_manager.side_effect = [mock_source_lab, mock_dest_lab]
        
        comparator = ProjectComparator(
            from_api_key="src_key",
            from_instance="https://api.test.ai",
            from_project_id="proj-1",
            from_organization_id="org-1",
            to_api_key="dest_key",
            to_instance="https://api.prod.ai",
            to_project_id="proj-2",
            to_organization_id="org-2",
            resource_types=["usage_limits"]
        )
        
        diff = comparator.compare()
        
        self.assertIn("usage_limits", diff.resource_diffs)
        limit_diff = diff.resource_diffs["usage_limits"]
        self.assertEqual(limit_diff.total_source, 0)
        self.assertEqual(limit_diff.total_destination, 0)

    @patch('pygeai.migration.comparison.SecretClient')
    @patch('pygeai.migration.comparison.AILabManager')
    def test_compare_secrets(self, mock_lab_manager, mock_secret_client):
        mock_source_lab = Mock()
        mock_dest_lab = Mock()
        mock_lab_manager.side_effect = [mock_source_lab, mock_dest_lab]
        
        mock_source_secret = Mock()
        mock_dest_secret = Mock()
        
        mock_source_secret.list_secrets.return_value = {
            "secrets": [{"id": "secret-1", "name": "Secret 1"}]
        }
        mock_dest_secret.list_secrets.return_value = {
            "secrets": []
        }
        
        mock_secret_client.side_effect = [mock_source_secret, mock_dest_secret]
        
        comparator = ProjectComparator(
            from_api_key="src_key",
            from_instance="https://api.test.ai",
            from_project_id="proj-1",
            from_organization_id="org-1",
            to_api_key="dest_key",
            to_instance="https://api.prod.ai",
            to_project_id="proj-2",
            to_organization_id="org-2",
            resource_types=["secrets"]
        )
        
        diff = comparator.compare()
        
        self.assertIn("secrets", diff.resource_diffs)
        secret_diff = diff.resource_diffs["secrets"]
        self.assertEqual(len(secret_diff.only_in_source), 1)

    @patch('pygeai.migration.comparison.SecretClient')
    @patch('pygeai.migration.comparison.AILabManager')
    def test_full_comparison_all_resources(self, mock_lab_manager, mock_secret_client):
        mock_source_lab = Mock()
        mock_dest_lab = Mock()
        
        mock_source_lab.list_agents.return_value = Mock(agents=[])
        mock_source_lab.list_tools.return_value = Mock(tools=[])
        mock_source_lab.list_processes.return_value = Mock(processes=[])
        mock_source_lab.list_tasks.return_value = Mock(tasks=[])
        mock_source_lab.list_artifact_types.return_value = Mock(artifact_types=[])
        
        mock_dest_lab.list_agents.return_value = Mock(agents=[])
        mock_dest_lab.list_tools.return_value = Mock(tools=[])
        mock_dest_lab.list_processes.return_value = Mock(processes=[])
        mock_dest_lab.list_tasks.return_value = Mock(tasks=[])
        mock_dest_lab.list_artifact_types.return_value = Mock(artifact_types=[])
        
        mock_lab_manager.side_effect = [mock_source_lab, mock_dest_lab]
        
        mock_source_secret = Mock()
        mock_dest_secret = Mock()
        mock_source_secret.list_secrets.return_value = {"secrets": []}
        mock_dest_secret.list_secrets.return_value = {"secrets": []}
        mock_secret_client.side_effect = [mock_source_secret, mock_dest_secret]
        
        comparator = ProjectComparator(
            from_api_key="src_key",
            from_instance="https://api.test.ai",
            from_project_id="proj-1",
            from_organization_id="org-1",
            to_api_key="dest_key",
            to_instance="https://api.prod.ai",
            to_project_id="proj-2",
            to_organization_id="org-2",
            resource_types=["agents", "tools", "processes", "tasks", "artifact_types", "secrets"]
        )
        
        diff = comparator.compare()
        
        self.assertEqual(len(diff.resource_diffs), 6)
        self.assertIn("agents", diff.resource_diffs)
        self.assertIn("tools", diff.resource_diffs)
        self.assertIn("processes", diff.resource_diffs)
        self.assertIn("tasks", diff.resource_diffs)
        self.assertIn("artifact_types", diff.resource_diffs)
        self.assertIn("secrets", diff.resource_diffs)

    @patch('pygeai.migration.comparison.AILabManager')
    def test_filtered_resource_types_agents_only(self, mock_lab_manager):
        mock_source_lab = Mock()
        mock_dest_lab = Mock()
        
        mock_source_lab.list_agents.return_value = Mock(agents=[])
        mock_dest_lab.list_agents.return_value = Mock(agents=[])
        
        mock_lab_manager.side_effect = [mock_source_lab, mock_dest_lab]
        
        comparator = ProjectComparator(
            from_api_key="src_key",
            from_instance="https://api.test.ai",
            from_project_id="proj-1",
            from_organization_id="org-1",
            to_api_key="dest_key",
            to_instance="https://api.prod.ai",
            to_project_id="proj-2",
            to_organization_id="org-2",
            resource_types=["agents"]
        )
        
        diff = comparator.compare()
        
        self.assertEqual(len(diff.resource_diffs), 1)
        self.assertIn("agents", diff.resource_diffs)
        self.assertNotIn("tools", diff.resource_diffs)


class TestDiffFormatter(unittest.TestCase):
    def setUp(self):
        self.formatter = DiffFormatter()
        self.sample_diff = ProjectDiff(
            source_project_id="proj-1",
            destination_project_id="proj-2",
            source_instance="https://api.test.ai",
            destination_instance="https://api.prod.ai",
            timestamp="2026-02-02T12:00:00",
            summary={"resource_types_compared": 2, "total_differences": 3}
        )
        
        self.sample_diff.resource_diffs["agents"] = ResourceDiff(
            resource_type="agents",
            only_in_source=[{"id": "agent-1", "name": "Agent 1"}],
            only_in_destination=[{"id": "agent-2", "name": "Agent 2"}],
            in_both=[{"id": "agent-3", "name": "Agent 3"}],
            total_source=2,
            total_destination=2
        )

    def test_format_as_text_basic(self):
        result = self.formatter.format_as_text(self.sample_diff)
        
        self.assertIn("PROJECT COMPARISON REPORT", result)
        self.assertIn("proj-1", result)
        self.assertIn("proj-2", result)
        self.assertIn("AGENTS", result)
        self.assertIn("Agent 1", result)
        self.assertIn("Agent 2", result)

    def test_format_as_text_no_differences(self):
        diff = ProjectDiff(
            source_project_id="proj-1",
            destination_project_id="proj-2",
            source_instance="https://api.test.ai",
            destination_instance="https://api.prod.ai",
            summary={"resource_types_compared": 0, "total_differences": 0}
        )
        
        result = self.formatter.format_as_text(diff)
        
        self.assertIn("Total differences found: 0", result)

    def test_format_as_json(self):
        result = self.formatter.format_as_json(self.sample_diff)
        
        data = json.loads(result)
        self.assertEqual(data["source_project_id"], "proj-1")
        self.assertEqual(data["destination_project_id"], "proj-2")
        self.assertIn("resource_diffs", data)
        self.assertIn("agents", data["resource_diffs"])

    def test_format_as_json_valid_structure(self):
        result = self.formatter.format_as_json(self.sample_diff)
        
        data = json.loads(result)
        self.assertIn("summary", data)
        self.assertIn("timestamp", data)
        self.assertIn("resource_diffs", data)
        self.assertEqual(data["summary"]["total_differences"], 3)

    def test_format_as_markdown(self):
        result = self.formatter.format_as_markdown(self.sample_diff)
        
        self.assertIn("# Project Comparison Report", result)
        self.assertIn("## Agents", result)
        self.assertIn("| Metric | Count |", result)
        self.assertIn("Agent 1", result)
        self.assertIn("Agent 2", result)

    def test_format_as_markdown_tables(self):
        result = self.formatter.format_as_markdown(self.sample_diff)
        
        self.assertIn("|--------|-------|", result)
        self.assertIn("| Source | 2 |", result)
        self.assertIn("| Destination | 2 |", result)

    def test_write_to_file(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            filepath = os.path.join(tmpdir, "test_report.txt")
            content = "Test content"
            
            self.formatter.write_to_file(content, filepath)
            
            self.assertTrue(os.path.exists(filepath))
            with open(filepath, 'r') as f:
                read_content = f.read()
            self.assertEqual(read_content, content)

    def test_write_to_file_creates_directory(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            filepath = os.path.join(tmpdir, "subdir", "test_report.txt")
            content = "Test content"
            
            self.formatter.write_to_file(content, filepath)
            
            self.assertTrue(os.path.exists(filepath))
            with open(filepath, 'r') as f:
                read_content = f.read()
            self.assertEqual(read_content, content)


if __name__ == '__main__':
    unittest.main()



import unittest
import warnings
import logging
import io
import tempfile
import os

from pygeai.core.base.session import Session, get_session, reset_session, _validate_alias
from pygeai.core.common.config import reset_settings, get_settings
from pygeai.core.common.constants import AuthType
from pygeai.core.common.exceptions import MissingRequirementException, MixedAuthenticationException


class TestSessionValidation(unittest.TestCase):
    """
    Tests for Session validation, warnings, and error handling.
    
    python -m unittest pygeai.tests.auth.test_session_validation.TestSessionValidation
    """

    def setUp(self):
        """Set up test fixtures"""
        reset_session()
        reset_settings()
        
        self.log_capture = io.StringIO()
        self.handler = logging.StreamHandler(self.log_capture)
        self.handler.setLevel(logging.WARNING)
        
        self.geai_logger = logging.getLogger('geai')
        self._original_handlers = self.geai_logger.handlers.copy()
        self._original_level = self.geai_logger.level
        
        self.geai_logger.handlers = []
        self.geai_logger.addHandler(self.handler)
        self.geai_logger.setLevel(logging.WARNING)
        self.geai_logger.propagate = False

    def tearDown(self):
        """Clean up test fixtures"""
        self.geai_logger.handlers = self._original_handlers
        self.geai_logger.level = self._original_level
        reset_session()
        reset_settings()

    def test_oauth_requires_project_id(self):
        """Test that OAuth access_token requires project_id"""
        with self.assertRaises(MissingRequirementException) as context:
            Session(
                base_url="https://api.test.com",
                access_token="oauth_token_123"
            )
        
        self.assertIn("project_id", str(context.exception).lower())

    def test_project_id_without_access_token_warns(self):
        """Test that project_id without access_token issues UserWarning"""
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            
            session = Session(
                api_key="api_key_123",
                base_url="https://api.test.com",
                project_id="project-123"
            )
            
            self.assertEqual(len(w), 1)
            self.assertTrue(issubclass(w[0].category, UserWarning))
            self.assertIn("project_id provided without access_token", str(w[0].message))

    def test_mixed_auth_without_allow_raises_error(self):
        """Test that both api_key and access_token without allow_mixed_auth raises MixedAuthenticationException"""
        with self.assertRaises(MixedAuthenticationException) as context:
            Session(
                api_key="api_key_123",
                base_url="https://api.test.com",
                access_token="oauth_token_123",
                project_id="project-456",
                allow_mixed_auth=False
            )
        
        self.assertIn("Cannot specify both", str(context.exception))

    def test_mixed_auth_with_allow_succeeds(self):
        """Test that mixed auth works when allow_mixed_auth=True"""
        session = Session(
            api_key="api_key_123",
            base_url="https://api.test.com",
            access_token="oauth_token_123",
            project_id="project-456",
            allow_mixed_auth=True
        )
        
        self.assertEqual(session.api_key, "api_key_123")
        self.assertEqual(session.access_token, "oauth_token_123")
        self.assertEqual(session.project_id, "project-456")
        self.assertEqual(session.auth_type, AuthType.OAUTH_TOKEN)

    def test_no_authentication_logs_warning(self):
        """Test that no authentication method logs warning"""
        Session(base_url="https://api.test.com")
        
        log_output = self.log_capture.getvalue()
        self.assertIn("No authentication method configured", log_output)

    def test_no_base_url_logs_warning(self):
        """Test that missing base_url logs warning"""
        Session(api_key="api_key_123")
        
        log_output = self.log_capture.getvalue()
        self.assertIn("Cannot instantiate session without base_url", log_output)

    def test_auth_type_api_key_only(self):
        """Test auth type is API_KEY when only api_key is provided"""
        session = Session(
            api_key="api_key_123",
            base_url="https://api.test.com"
        )
        
        self.assertEqual(session.auth_type, AuthType.API_KEY)
        self.assertTrue(session.is_api_key())
        self.assertFalse(session.is_oauth())

    def test_auth_type_oauth_only(self):
        """Test auth type is OAUTH_TOKEN when OAuth credentials are provided"""
        session = Session(
            base_url="https://api.test.com",
            access_token="oauth_token_123",
            project_id="project-456"
        )
        
        self.assertEqual(session.auth_type, AuthType.OAUTH_TOKEN)
        self.assertTrue(session.is_oauth())
        self.assertFalse(session.is_api_key())

    def test_auth_type_none(self):
        """Test auth type is NONE when no credentials are provided"""
        session = Session(base_url="https://api.test.com")
        
        self.assertEqual(session.auth_type, AuthType.NONE)
        self.assertFalse(session.is_oauth())
        self.assertFalse(session.is_api_key())

    def test_auth_type_oauth_takes_precedence(self):
        """Test that OAuth takes precedence over API key when both are present"""
        session = Session(
            api_key="api_key_123",
            base_url="https://api.test.com",
            access_token="oauth_token_123",
            project_id="project-456",
            allow_mixed_auth=True
        )
        
        self.assertEqual(session.auth_type, AuthType.OAUTH_TOKEN)

    def test_get_active_token_api_key(self):
        """Test get_active_token returns api_key when using API key auth"""
        session = Session(
            api_key="api_key_123",
            base_url="https://api.test.com"
        )
        
        self.assertEqual(session.get_active_token(), "api_key_123")

    def test_get_active_token_oauth(self):
        """Test get_active_token returns access_token when using OAuth"""
        session = Session(
            base_url="https://api.test.com",
            access_token="oauth_token_123",
            project_id="project-456"
        )
        
        self.assertEqual(session.get_active_token(), "oauth_token_123")

    def test_get_active_token_none(self):
        """Test get_active_token returns None when no auth is configured"""
        session = Session(base_url="https://api.test.com")
        
        self.assertIsNone(session.get_active_token())

    def test_setter_updates_auth_type_api_key(self):
        """Test that setting api_key updates auth type"""
        session = Session(base_url="https://api.test.com")
        self.assertEqual(session.auth_type, AuthType.NONE)
        
        session.api_key = "new_api_key"
        self.assertEqual(session.auth_type, AuthType.API_KEY)

    def test_setter_updates_auth_type_oauth(self):
        """Test that setting OAuth properties updates auth type"""
        session = Session(
            api_key="api_key_123",
            base_url="https://api.test.com"
        )
        self.assertEqual(session.auth_type, AuthType.API_KEY)
        
        session.access_token = "oauth_token_123"
        session.project_id = "project-456"
        self.assertEqual(session.auth_type, AuthType.OAUTH_TOKEN)

    def test_alias_defaults_to_default(self):
        """Test that alias defaults to 'default' when not provided"""
        session = Session(
            api_key="api_key_123",
            base_url="https://api.test.com"
        )
        
        self.assertEqual(session.alias, "default")

    def test_alias_can_be_set(self):
        """Test that custom alias can be set"""
        session = Session(
            api_key="api_key_123",
            base_url="https://api.test.com",
            alias="production"
        )
        
        self.assertEqual(session.alias, "production")

    def test_organization_id_can_be_set(self):
        """Test that organization_id can be set and retrieved"""
        session = Session(
            base_url="https://api.test.com",
            access_token="oauth_token_123",
            project_id="project-456",
            organization_id="org-789"
        )
        
        self.assertEqual(session.organization_id, "org-789")


class TestGetSession(unittest.TestCase):
    """
    Tests for get_session function and singleton behavior.
    
    python -m unittest pygeai.tests.auth.test_session_validation.TestGetSession
    """

    def setUp(self):
        """Set up test fixtures"""
        reset_session()
        reset_settings()
        
        self.log_capture = io.StringIO()
        self.handler = logging.StreamHandler(self.log_capture)
        self.handler.setLevel(logging.WARNING)
        
        self.geai_logger = logging.getLogger('geai')
        self._original_handlers = self.geai_logger.handlers.copy()
        self._original_level = self.geai_logger.level
        
        self.geai_logger.handlers = []
        self.geai_logger.addHandler(self.handler)
        self.geai_logger.setLevel(logging.WARNING)
        self.geai_logger.propagate = False

    def tearDown(self):
        """Clean up test fixtures"""
        self.geai_logger.handlers = self._original_handlers
        self.geai_logger.level = self._original_level
        reset_session()
        reset_settings()

    def test_get_session_is_singleton(self):
        """Test that get_session returns the same instance"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='_credentials', delete=False) as f:
            creds_file = f.name
            f.write('[default]\n')
            f.write('GEAI_API_KEY = test_key\n')
            f.write('GEAI_API_BASE_URL = https://test.example.com\n')
        
        try:
            reset_settings()
            get_settings(credentials_file=creds_file)
            
            session1 = get_session()
            session2 = get_session()
            
            self.assertIs(session1, session2)
        finally:
            os.unlink(creds_file)

    def test_get_session_loads_from_config(self):
        """Test that get_session loads credentials from config file"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='_credentials', delete=False) as f:
            creds_file = f.name
            f.write('[default]\n')
            f.write('GEAI_API_KEY = config_key_123\n')
            f.write('GEAI_API_BASE_URL = https://config.example.com\n')
        
        try:
            reset_settings()
            get_settings(credentials_file=creds_file)
            
            session = get_session()
            
            self.assertEqual(session.api_key, "config_key_123")
            self.assertEqual(session.base_url, "https://config.example.com")
        finally:
            os.unlink(creds_file)

    def test_get_session_loads_oauth_from_config(self):
        """Test that get_session loads OAuth credentials from config"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='_credentials', delete=False) as f:
            creds_file = f.name
            f.write('[default]\n')
            f.write('GEAI_OAUTH_ACCESS_TOKEN = oauth_token_123\n')
            f.write('GEAI_PROJECT_ID = project-456\n')
            f.write('GEAI_API_BASE_URL = https://oauth.example.com\n')
        
        try:
            reset_settings()
            get_settings(credentials_file=creds_file)
            
            session = get_session()
            
            self.assertEqual(session.access_token, "oauth_token_123")
            self.assertEqual(session.project_id, "project-456")
            self.assertEqual(session.auth_type, AuthType.OAUTH_TOKEN)
        finally:
            os.unlink(creds_file)

    def test_get_session_warns_about_mixed_auth(self):
        """Test that get_session warns when both API key and OAuth are configured"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='_credentials', delete=False) as f:
            creds_file = f.name
            f.write('[default]\n')
            f.write('GEAI_API_KEY = api_key_123\n')
            f.write('GEAI_OAUTH_ACCESS_TOKEN = oauth_token_123\n')
            f.write('GEAI_PROJECT_ID = project-456\n')
            f.write('GEAI_API_BASE_URL = https://mixed.example.com\n')
        
        try:
            reset_settings()
            get_settings(credentials_file=creds_file)
            
            session = get_session()
            
            log_output = self.log_capture.getvalue()
            self.assertIn("Both API key and OAuth token configured", log_output)
            self.assertIn("OAuth token will take precedence", log_output)
        finally:
            os.unlink(creds_file)

    def test_get_session_updates_existing_with_alias(self):
        """Test that get_session with alias updates existing session"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='_credentials', delete=False) as f:
            creds_file = f.name
            f.write('[default]\n')
            f.write('GEAI_API_KEY = default_key\n')
            f.write('GEAI_API_BASE_URL = https://default.example.com\n')
            f.write('\n')
            f.write('[staging]\n')
            f.write('GEAI_API_KEY = staging_key\n')
            f.write('GEAI_API_BASE_URL = https://staging.example.com\n')
        
        try:
            reset_settings()
            get_settings(credentials_file=creds_file)
            
            session1 = get_session('default')
            self.assertEqual(session1.api_key, "default_key")
            
            session2 = get_session('staging')
            self.assertEqual(session2.api_key, "staging_key")
            
            self.assertIs(session1, session2)
        finally:
            os.unlink(creds_file)

    def test_validate_alias_raises_for_missing(self):
        """Test that _validate_alias raises exception for non-existent alias"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='_credentials', delete=False) as f:
            creds_file = f.name
            f.write('[default]\n')
            f.write('GEAI_API_KEY = test_key\n')
            f.write('GEAI_API_BASE_URL = https://test.example.com\n')
        
        try:
            reset_settings()
            get_settings(credentials_file=creds_file)
            
            with self.assertRaises(MissingRequirementException) as context:
                _validate_alias('nonexistent', allow_missing_default=False)
            
            self.assertIn("doesn't exist", str(context.exception))
        finally:
            os.unlink(creds_file)

    def test_validate_alias_allows_missing_default(self):
        """Test that _validate_alias allows missing 'default' when flag is set"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='_credentials', delete=False) as f:
            creds_file = f.name
            f.write('[production]\n')
            f.write('GEAI_API_KEY = prod_key\n')
            f.write('GEAI_API_BASE_URL = https://prod.example.com\n')
        
        try:
            reset_settings()
            get_settings(credentials_file=creds_file)
            
            _validate_alias('default', allow_missing_default=True)
        finally:
            os.unlink(creds_file)


if __name__ == '__main__':
    unittest.main()



from typing import List

from pygeai.core.omni.responses import OmniParserResponse, OmniParserPart, OmniParserUsage


class OmniParserResponseMapper:

    @classmethod
    def map_to_omni_parser_response(cls, data: dict) -> OmniParserResponse:
        parts_list = cls.map_to_part_list(data.get('parts', []))
        usage = cls.map_to_usage(data.get('usage'))

        return OmniParserResponse(
            status=data.get('status', ''),
            parts=parts_list,
            request_id=data.get('requestId', ''),
            usage=usage
        )

    @classmethod
    def map_to_part_list(cls, data: list) -> List[OmniParserPart]:
        part_list = []
        for item in data:
            part = cls.map_to_part(item)
            part_list.append(part)
        return part_list

    @classmethod
    def map_to_part(cls, data: dict) -> OmniParserPart:
        return OmniParserPart(
            page=data.get('page'),
            type=data.get('type'),
            text=data.get('text', ''),
            mid_time=data.get('midTime')
        )

    @classmethod
    def map_to_usage(cls, data: dict) -> OmniParserUsage | None:
        if data is None:
            return None

        return OmniParserUsage(
            total_cost=data.get('total_cost'),
            total_tokens=data.get('total_tokens'),
            completion_tokens=data.get('completion_tokens'),
            prompt_tokens=data.get('prompt_tokens'),
            prompt_cost=data.get('prompt_cost'),
            completion_cost=data.get('completion_cost'),
            currency=data.get('currency'),
            prompt_tokens_details=data.get('prompt_tokens_details'),
            completion_tokens_details=data.get('completion_tokens_details')
        )



import unittest
from unittest.mock import patch, MagicMock
from pygeai.cli.commands.chat import get_chat_completion, chat_with_iris, chat_with_agent
from pygeai.core.common.exceptions import MissingRequirementException, WrongArgumentError
from pygeai.cli.commands import Option


class TestChatCommands(unittest.TestCase):
    """
    python -m unittest pygeai.tests.cli.commands.test_chat.TestChatCommands
    """

    @patch('pygeai.cli.commands.chat.ChatClient')
    @patch('pygeai.cli.commands.common.get_messages')
    def test_get_chat_completion_success(self, mock_get_messages, mock_chat_client):
        mock_instance = mock_chat_client.return_value
        mock_instance.chat_completion.return_value = "Chat response"
        mock_get_messages.return_value = [{"role": "user", "content": "Hello"}]

        option_list = [
            (Option("model", ["--model"], "", True), "test-model"),
            (Option("messages", ["--messages"], "", True), '{"role": "user", "content": "Hello"}')
        ]

        get_chat_completion(option_list)
        mock_chat_client.assert_called_once()
        mock_instance.chat_completion.assert_called_once()

    def test_get_chat_completion_missing_requirements(self):
        option_list = [
            (Option("model", ["--model"], "", True), "test-model")
        ]
        with self.assertRaises(MissingRequirementException):
            get_chat_completion(option_list)

    def test_get_chat_completion_invalid_messages_format(self):
        option_list = [
            (Option("model", ["--model"], "", True), "test-model"),
            (Option("messages", ["--messages"], "", True), "invalid-json")
        ]
        with self.assertRaises(WrongArgumentError):
            get_chat_completion(option_list)

    @patch('pygeai.cli.commands.chat.Iris')
    @patch('prompt_toolkit.PromptSession.prompt')
    def test_chat_with_iris_success(self, mock_prompt, mock_iris):
        mock_prompt.side_effect = ["Hello", "exit"]
        mock_iris_instance = mock_iris.return_value
        mock_iris_instance.stream_answer.return_value = iter(["Hi there!"])

        chat_with_iris()
        mock_iris.assert_called_once()
        mock_iris_instance.stream_answer.assert_called()

    @patch('pygeai.cli.commands.chat.get_project_id')
    @patch('pygeai.cli.commands.chat.AgentClient')
    @patch('pygeai.cli.commands.chat.AgentChatSession')
    @patch('prompt_toolkit.PromptSession.prompt')
    def test_chat_with_agent_success(self, mock_prompt, mock_agent_session, mock_agent_client, mock_get_project_id):
        mock_prompt.side_effect = ["Hello", "exit"]
        mock_session_instance = mock_agent_session.return_value
        mock_session_instance.get_answer.return_value = "Hello, I'm your agent!"
        mock_session_instance.stream_answer.return_value = iter(["Hi there!"])
        
        mock_get_project_id.return_value = "test-project-id"
        mock_client_instance = mock_agent_client.return_value
        mock_client_instance.get_agent.return_value = {"agentId": "123", "agentName": "test-agent"}

        option_list = [
            (Option("agent_name", ["--agent-name"], "", True), "test-agent")
        ]

        chat_with_agent(option_list)
        mock_agent_session.assert_called_once_with("test-agent")
        mock_session_instance.stream_answer.assert_called()

    def test_chat_with_agent_missing_agent_name(self):
        option_list = []
        with self.assertRaises(MissingRequirementException):
            chat_with_agent(option_list)

    @patch('pygeai.cli.commands.chat.get_project_id')
    @patch('pygeai.cli.commands.chat.AgentClient')
    @patch('pygeai.cli.commands.chat.AgentChatSession')
    @patch('prompt_toolkit.PromptSession.prompt')
    @patch('builtins.open', new_callable=MagicMock)
    @patch('json.load')
    def test_chat_with_agent_restore_session_success(self, mock_json_load, mock_open, mock_prompt, mock_agent_session, mock_agent_client, mock_get_project_id):
        mock_prompt.side_effect = ["Hello", "exit"]
        mock_session_instance = mock_agent_session.return_value
        mock_session_instance.get_answer.return_value = "Hello, I'm your agent!"
        mock_session_instance.stream_answer.return_value = iter(["Hi there!"])
        mock_json_load.return_value = [{"role": "user", "content": "Hi"}, {"role": "assistant", "content": "Hello!"}]
        
        mock_get_project_id.return_value = "test-project-id"
        mock_client_instance = mock_agent_client.return_value
        mock_client_instance.get_agent.return_value = {"agentId": "123", "agentName": "test-agent"}

        option_list = [
            (Option("agent_name", ["--agent-name"], "", True), "test-agent"),
            (Option("restore_session", ["--restore-session"], "", True), "session.json")
        ]

        chat_with_agent(option_list)
        mock_json_load.assert_called_once()

    @patch('pygeai.cli.commands.chat.get_project_id')
    @patch('pygeai.cli.commands.chat.AgentClient')
    @patch('pygeai.cli.commands.chat.AgentChatSession')
    @patch('prompt_toolkit.PromptSession.prompt')
    @patch('builtins.open', new_callable=MagicMock)
    @patch('json.dump')
    def test_chat_with_agent_save_session_success(self, mock_json_dump, mock_open, mock_prompt, mock_agent_session, mock_agent_client, mock_get_project_id):
        mock_prompt.side_effect = ["Hello", "exit"]
        mock_session_instance = mock_agent_session.return_value
        mock_session_instance.get_answer.return_value = "Hello, I'm your agent!"
        mock_session_instance.stream_answer.return_value = iter(["Hi there!"])
        
        mock_get_project_id.return_value = "test-project-id"
        mock_client_instance = mock_agent_client.return_value
        mock_client_instance.get_agent.return_value = {"agentId": "123", "agentName": "test-agent"}

        option_list = [
            (Option("agent_name", ["--agent-name"], "", True), "test-agent"),
            (Option("save_session", ["--save-session"], "", True), "session.json")
        ]

        chat_with_agent(option_list)
        mock_json_dump.assert_called()




import logging

from collections.abc import Awaitable, Callable
from typing import TYPE_CHECKING, Any


if TYPE_CHECKING:
    from starlette.applications import Starlette
    from starlette.routing import Route

    _package_starlette_installed = True

else:
    try:
        from starlette.applications import Starlette
        from starlette.routing import Route

        _package_starlette_installed = True
    except ImportError:
        Starlette = Any
        Route = Any

        _package_starlette_installed = False

from a2a.server.apps.jsonrpc.jsonrpc_app import (
    CallContextBuilder,
    JSONRPCApplication,
)
from a2a.server.context import ServerCallContext
from a2a.server.request_handlers.jsonrpc_handler import RequestHandler
from a2a.types import AgentCard
from a2a.utils.constants import (
    AGENT_CARD_WELL_KNOWN_PATH,
    DEFAULT_RPC_URL,
    EXTENDED_AGENT_CARD_PATH,
    PREV_AGENT_CARD_WELL_KNOWN_PATH,
)


logger = logging.getLogger(__name__)


class A2AStarletteApplication(JSONRPCApplication):
    """A Starlette application implementing the A2A protocol server endpoints.

    Handles incoming JSON-RPC requests, routes them to the appropriate
    handler methods, and manages response generation including Server-Sent Events
    (SSE).
    """

    def __init__(  # noqa: PLR0913
        self,
        agent_card: AgentCard,
        http_handler: RequestHandler,
        extended_agent_card: AgentCard | None = None,
        context_builder: CallContextBuilder | None = None,
        card_modifier: Callable[[AgentCard], Awaitable[AgentCard] | AgentCard]
        | None = None,
        extended_card_modifier: Callable[
            [AgentCard, ServerCallContext], Awaitable[AgentCard] | AgentCard
        ]
        | None = None,
        max_content_length: int | None = 10 * 1024 * 1024,  # 10MB
    ) -> None:
        """Initializes the A2AStarletteApplication.

        Args:
            agent_card: The AgentCard describing the agent's capabilities.
            http_handler: The handler instance responsible for processing A2A
              requests via http.
            extended_agent_card: An optional, distinct AgentCard to be served
              at the authenticated extended card endpoint.
            context_builder: The CallContextBuilder used to construct the
              ServerCallContext passed to the http_handler. If None, no
              ServerCallContext is passed.
            card_modifier: An optional callback to dynamically modify the public
              agent card before it is served.
            extended_card_modifier: An optional callback to dynamically modify
              the extended agent card before it is served. It receives the
              call context.
            max_content_length: The maximum allowed content length for incoming
              requests. Defaults to 10MB. Set to None for unbounded maximum.
        """
        if not _package_starlette_installed:
            raise ImportError(
                'Packages `starlette` and `sse-starlette` are required to use the'
                ' `A2AStarletteApplication`. It can be added as a part of `a2a-sdk`'
                ' optional dependencies, `a2a-sdk[http-server]`.'
            )
        super().__init__(
            agent_card=agent_card,
            http_handler=http_handler,
            extended_agent_card=extended_agent_card,
            context_builder=context_builder,
            card_modifier=card_modifier,
            extended_card_modifier=extended_card_modifier,
            max_content_length=max_content_length,
        )

    def routes(
        self,
        agent_card_url: str = AGENT_CARD_WELL_KNOWN_PATH,
        rpc_url: str = DEFAULT_RPC_URL,
        extended_agent_card_url: str = EXTENDED_AGENT_CARD_PATH,
    ) -> list[Route]:
        """Returns the Starlette Routes for handling A2A requests.

        Args:
            agent_card_url: The URL path for the agent card endpoint.
            rpc_url: The URL path for the A2A JSON-RPC endpoint (POST requests).
            extended_agent_card_url: The URL for the authenticated extended agent card endpoint.

        Returns:
            A list of Starlette Route objects.
        """
        app_routes = [
            Route(
                rpc_url,
                self._handle_requests,
                methods=['POST'],
                name='a2a_handler',
            ),
            Route(
                agent_card_url,
                self._handle_get_agent_card,
                methods=['GET'],
                name='agent_card',
            ),
        ]

        if agent_card_url == AGENT_CARD_WELL_KNOWN_PATH:
            # For backward compatibility, serve the agent card at the deprecated path as well.
            # TODO: remove in a future release
            app_routes.append(
                Route(
                    PREV_AGENT_CARD_WELL_KNOWN_PATH,
                    self._handle_get_agent_card,
                    methods=['GET'],
                    name='deprecated_agent_card',
                )
            )

        # TODO: deprecated endpoint to be removed in a future release
        if self.agent_card.supports_authenticated_extended_card:
            app_routes.append(
                Route(
                    extended_agent_card_url,
                    self._handle_get_authenticated_extended_agent_card,
                    methods=['GET'],
                    name='authenticated_extended_agent_card',
                )
            )
        return app_routes

    def add_routes_to_app(
        self,
        app: Starlette,
        agent_card_url: str = AGENT_CARD_WELL_KNOWN_PATH,
        rpc_url: str = DEFAULT_RPC_URL,
        extended_agent_card_url: str = EXTENDED_AGENT_CARD_PATH,
    ) -> None:
        """Adds the routes to the Starlette application.

        Args:
            app: The Starlette application to add the routes to.
            agent_card_url: The URL path for the agent card endpoint.
            rpc_url: The URL path for the A2A JSON-RPC endpoint (POST requests).
            extended_agent_card_url: The URL for the authenticated extended agent card endpoint.
        """
        routes = self.routes(
            agent_card_url=agent_card_url,
            rpc_url=rpc_url,
            extended_agent_card_url=extended_agent_card_url,
        )
        app.routes.extend(routes)

    def build(
        self,
        agent_card_url: str = AGENT_CARD_WELL_KNOWN_PATH,
        rpc_url: str = DEFAULT_RPC_URL,
        extended_agent_card_url: str = EXTENDED_AGENT_CARD_PATH,
        **kwargs: Any,
    ) -> Starlette:
        """Builds and returns the Starlette application instance.

        Args:
            agent_card_url: The URL path for the agent card endpoint.
            rpc_url: The URL path for the A2A JSON-RPC endpoint (POST requests).
            extended_agent_card_url: The URL for the authenticated extended agent card endpoint.
            **kwargs: Additional keyword arguments to pass to the Starlette constructor.

        Returns:
            A configured Starlette application instance.
        """
        app = Starlette(**kwargs)

        self.add_routes_to_app(
            app, agent_card_url, rpc_url, extended_agent_card_url
        )

        return app



import unittest
from unittest.mock import Mock, patch

from pygeai.core.base.models import Error
from pygeai.migration.strategies import (
    ProjectMigrationStrategy,
    AgentMigrationStrategy,
    ToolMigrationStrategy,
    AgenticProcessMigrationStrategy,
    TaskMigrationStrategy,
    ArtifactTypeMigrationStrategy,
)
from pygeai.core.models import Project
from pygeai.core.base.responses import ErrorListResponse
from pygeai.lab.models import Agent, Tool, AgenticProcess, Task, ArtifactTypeDefinition


class TestMigrationStrategies(unittest.TestCase):
    """
    python -m unittest pygeai.tests.migration.test_strategies.TestMigrationStrategies
    """

    def setUp(self):
        self.from_api_key = "from_key"
        self.from_instance = "http://from.instance"
        self.to_api_key = "to_key"
        self.to_instance = "http://to.instance"
        self.from_project_id = "proj_123"
        self.to_project_name = "New Project"
        self.admin_email = "admin@example.com"
        self.agent_id = "agent_123"
        self.tool_id = "tool_123"
        self.process_id = "process_123"
        self.task_id = "task_123"
        self.artifact_type_id = "artifact_type_123"

    @patch('pygeai.migration.strategies.OrganizationManager')
    def test_project_migration_strategy_init(self, mock_org_manager):
        strategy = ProjectMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            from_project_id=self.from_project_id,
            to_project_name=self.to_project_name,
            admin_email=self.admin_email
        )
        self.assertEqual(strategy.from_api_key, self.from_api_key)
        self.assertEqual(strategy.from_instance, self.from_instance)
        self.assertEqual(strategy.to_api_key, self.to_api_key)
        self.assertEqual(strategy.to_instance, self.to_instance)
        self.assertEqual(strategy.from_project_id, self.from_project_id)
        self.assertEqual(strategy.to_project_name, self.to_project_name)
        self.assertEqual(strategy.admin_email, self.admin_email)
        mock_org_manager.assert_any_call(api_key=self.from_api_key, base_url=self.from_instance)
        mock_org_manager.assert_any_call(api_key=self.to_api_key, base_url=self.to_instance)

    @patch('pygeai.migration.strategies.OrganizationManager')
    def test_project_migration_success(self, mock_org_manager):
        mock_source = Mock()
        mock_destination = Mock()
        mock_org_manager.side_effect = [mock_source, mock_destination]

        mock_project = Mock(spec=Project, name="Old Project", email="old@example.com")
        mock_new_project = Mock(spec=Project, name=self.to_project_name, email=self.admin_email)
        mock_response = Mock(project=mock_new_project)

        mock_source.get_project_data.return_value = Mock(project=mock_project)
        mock_destination.create_project.return_value = mock_response

        strategy = ProjectMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            from_project_id=self.from_project_id,
            to_project_name=self.to_project_name,
            admin_email=self.admin_email
        )
        strategy.migrate()

        mock_source.get_project_data.assert_called_with(project_id=self.from_project_id)
        mock_destination.create_project.assert_called_with(mock_project)

    @patch('pygeai.migration.strategies.OrganizationManager')
    def test_project_migration_error_no_project_data(self, mock_org_manager):
        mock_source = Mock()
        mock_destination = Mock()
        mock_org_manager.side_effect = [mock_source, mock_destination]

        mock_source.get_project_data.return_value = Mock(spec=[])

        strategy = ProjectMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            from_project_id=self.from_project_id,
            to_project_name=self.to_project_name,
            admin_email=self.admin_email
        )

        with self.assertRaises(ValueError) as context:
            strategy.migrate()
        self.assertIn("Unable to retrieve project data", str(context.exception))
        mock_destination.create_project.assert_not_called()

    @patch('pygeai.migration.strategies.OrganizationManager')
    def test_project_migration_error_response(self, mock_org_manager):
        mock_source = Mock()
        mock_destination = Mock()
        mock_org_manager.side_effect = [mock_source, mock_destination]

        mock_project = Mock(spec=Project)
        mock_source.get_project_data.return_value = Mock(project=mock_project)
        mock_error_response = ErrorListResponse(errors=[Error(id=1, description="Creation failed")])
        mock_destination.create_project.return_value = mock_error_response

        strategy = ProjectMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            from_project_id=self.from_project_id,
            to_project_name=self.to_project_name,
            admin_email=self.admin_email
        )

        with self.assertRaises(ValueError) as context:
            strategy.migrate()
        self.assertIn("Failed to create project", str(context.exception))

    @patch('pygeai.migration.strategies.AILabManager')
    def test_agent_migration_strategy_init(self, mock_lab_manager):
        strategy = AgentMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            agent_id=self.agent_id
        )
        self.assertEqual(strategy.from_api_key, self.from_api_key)
        self.assertEqual(strategy.from_instance, self.from_instance)
        self.assertEqual(strategy.to_api_key, self.to_api_key)
        self.assertEqual(strategy.to_instance, self.to_instance)
        self.assertEqual(strategy.agent_id, self.agent_id)
        mock_lab_manager.assert_any_call(api_key=self.from_api_key, base_url=self.from_instance)
        mock_lab_manager.assert_any_call(api_key=self.to_api_key, base_url=self.to_instance)

    @patch('pygeai.core.utils.console.Console.write_stdout')
    @patch('pygeai.migration.strategies.AILabManager')
    def test_agent_migration_success(self, mock_lab_manager, mock_stdout):
        mock_source = Mock()
        mock_destination = Mock()
        mock_lab_manager.side_effect = [mock_source, mock_destination]

        mock_agent = Mock(spec=Agent)
        mock_source.get_agent.return_value = mock_agent
        mock_destination.create_agent.return_value = mock_agent

        strategy = AgentMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            agent_id=self.agent_id
        )
        strategy.migrate()

        mock_source.get_agent.assert_called_with(agent_id=self.agent_id)
        mock_destination.create_agent.assert_called_with(agent=mock_agent)
        mock_stdout.assert_called_once()
        self.assertIn("New agent detail:", mock_stdout.call_args[0][0])

    @patch('pygeai.migration.strategies.AILabManager')
    def test_agent_migration_error_not_agent(self, mock_lab_manager):
        mock_source = Mock()
        mock_destination = Mock()
        mock_lab_manager.side_effect = [mock_source, mock_destination]

        mock_source.get_agent.return_value = Mock(spec=[])

        strategy = AgentMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            agent_id=self.agent_id
        )

        with self.assertRaises(ValueError) as context:
            strategy.migrate()
        self.assertIn("Agent migration failed", str(context.exception))
        mock_destination.create_agent.assert_not_called()

    @patch('pygeai.migration.strategies.AILabManager')
    def test_agent_migration_error_exception(self, mock_lab_manager):
        mock_source = Mock()
        mock_destination = Mock()
        mock_lab_manager.side_effect = [mock_source, mock_destination]

        mock_source.get_agent.side_effect = Exception("API Error")

        strategy = AgentMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            agent_id=self.agent_id
        )

        with self.assertRaises(ValueError) as context:
            strategy.migrate()
        self.assertIn("Agent migration failed", str(context.exception))

    @patch('pygeai.migration.strategies.AILabManager')
    def test_tool_migration_strategy_init(self, mock_lab_manager):
        strategy = ToolMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            tool_id=self.tool_id
        )
        self.assertEqual(strategy.from_api_key, self.from_api_key)
        self.assertEqual(strategy.from_instance, self.from_instance)
        self.assertEqual(strategy.to_api_key, self.to_api_key)
        self.assertEqual(strategy.to_instance, self.to_instance)
        self.assertEqual(strategy.tool_id, self.tool_id)
        mock_lab_manager.assert_any_call(api_key=self.from_api_key, base_url=self.from_instance)
        mock_lab_manager.assert_any_call(api_key=self.to_api_key, base_url=self.to_instance)

    @patch('pygeai.core.utils.console.Console.write_stdout')
    @patch('pygeai.migration.strategies.AILabManager')
    def test_tool_migration_success(self, mock_lab_manager, mock_stdout):
        mock_source = Mock()
        mock_destination = Mock()
        mock_lab_manager.side_effect = [mock_source, mock_destination]

        mock_tool = Mock(spec=Tool)
        mock_source.get_tool.return_value = mock_tool
        mock_destination.create_tool.return_value = mock_tool

        strategy = ToolMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            tool_id=self.tool_id
        )
        strategy.migrate()

        mock_source.get_tool.assert_called_with(tool_id=self.tool_id)
        mock_destination.create_tool.assert_called_with(tool=mock_tool)
        mock_stdout.assert_called_once()
        self.assertIn("New tool detail:", mock_stdout.call_args[0][0])

    @patch('pygeai.migration.strategies.AILabManager')
    def test_tool_migration_error_not_tool(self, mock_lab_manager):
        mock_source = Mock()
        mock_destination = Mock()
        mock_lab_manager.side_effect = [mock_source, mock_destination]

        mock_source.get_tool.return_value = Mock(spec=[])

        strategy = ToolMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            tool_id=self.tool_id
        )

        with self.assertRaises(ValueError) as context:
            strategy.migrate()
        self.assertIn("Tool migration failed", str(context.exception))
        mock_destination.create_tool.assert_not_called()

    @patch('pygeai.migration.strategies.AILabManager')
    def test_process_migration_strategy_init(self, mock_lab_manager):
        strategy = AgenticProcessMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            process_id=self.process_id
        )
        self.assertEqual(strategy.from_api_key, self.from_api_key)
        self.assertEqual(strategy.from_instance, self.from_instance)
        self.assertEqual(strategy.to_api_key, self.to_api_key)
        self.assertEqual(strategy.to_instance, self.to_instance)
        self.assertEqual(strategy.process_id, self.process_id)
        mock_lab_manager.assert_any_call(api_key=self.from_api_key, base_url=self.from_instance)
        mock_lab_manager.assert_any_call(api_key=self.to_api_key, base_url=self.to_instance)

    @patch('pygeai.core.utils.console.Console.write_stdout')
    @patch('pygeai.migration.strategies.AILabManager')
    def test_process_migration_success(self, mock_lab_manager, mock_stdout):
        mock_source = Mock()
        mock_destination = Mock()
        mock_lab_manager.side_effect = [mock_source, mock_destination]

        mock_process = Mock(spec=AgenticProcess)
        mock_source.get_process.return_value = mock_process
        mock_destination.create_process.return_value = mock_process

        strategy = AgenticProcessMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            process_id=self.process_id
        )
        strategy.migrate()

        mock_source.get_process.assert_called_with(process_id=self.process_id)
        mock_destination.create_process.assert_called_with(process=mock_process)
        mock_stdout.assert_called_once()
        self.assertIn("New process detail:", mock_stdout.call_args[0][0])

    @patch('pygeai.migration.strategies.AILabManager')
    def test_process_migration_error_not_process(self, mock_lab_manager):
        mock_source = Mock()
        mock_destination = Mock()
        mock_lab_manager.side_effect = [mock_source, mock_destination]

        mock_source.get_process.return_value = Mock(spec=[])

        strategy = AgenticProcessMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            process_id=self.process_id
        )

        with self.assertRaises(ValueError) as context:
            strategy.migrate()
        self.assertIn("Process migration failed", str(context.exception))
        mock_destination.create_process.assert_not_called()

    @patch('pygeai.migration.strategies.AILabManager')
    def test_task_migration_strategy_init(self, mock_lab_manager):
        strategy = TaskMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            task_id=self.task_id
        )
        self.assertEqual(strategy.from_api_key, self.from_api_key)
        self.assertEqual(strategy.from_instance, self.from_instance)
        self.assertEqual(strategy.to_api_key, self.to_api_key)
        self.assertEqual(strategy.to_instance, self.to_instance)
        self.assertEqual(strategy.task_id, self.task_id)
        mock_lab_manager.assert_any_call(api_key=self.from_api_key, base_url=self.from_instance)
        mock_lab_manager.assert_any_call(api_key=self.to_api_key, base_url=self.to_instance)

    @patch('pygeai.core.utils.console.Console.write_stdout')
    @patch('pygeai.migration.strategies.AILabManager')
    def test_task_migration_success(self, mock_lab_manager, mock_stdout):
        mock_source = Mock()
        mock_destination = Mock()
        mock_lab_manager.side_effect = [mock_source, mock_destination]

        mock_task = Mock(spec=Task)
        mock_source.get_task.return_value = mock_task
        mock_destination.create_task.return_value = mock_task

        strategy = TaskMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            task_id=self.task_id
        )
        strategy.migrate()

        mock_source.get_task.assert_called_with(task_id=self.task_id)
        mock_destination.create_task.assert_called_with(task=mock_task)
        mock_stdout.assert_called_once()
        self.assertIn("New task detail:", mock_stdout.call_args[0][0])

    @patch('pygeai.migration.strategies.AILabManager')
    def test_task_migration_error_not_task(self, mock_lab_manager):
        mock_source = Mock()
        mock_destination = Mock()
        mock_lab_manager.side_effect = [mock_source, mock_destination]

        mock_source.get_task.return_value = Mock(spec=[])

        strategy = TaskMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            task_id=self.task_id
        )

        with self.assertRaises(ValueError) as context:
            strategy.migrate()
        self.assertIn("Task migration failed", str(context.exception))
        mock_destination.create_task.assert_not_called()

    @patch('pygeai.migration.strategies.AILabManager')
    def test_artifact_type_migration_strategy_init(self, mock_lab_manager):
        strategy = ArtifactTypeMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            artifact_type_id=self.artifact_type_id
        )
        self.assertEqual(strategy.from_api_key, self.from_api_key)
        self.assertEqual(strategy.from_instance, self.from_instance)
        self.assertEqual(strategy.to_api_key, self.to_api_key)
        self.assertEqual(strategy.to_instance, self.to_instance)
        self.assertEqual(strategy.artifact_type_id, self.artifact_type_id)
        mock_lab_manager.assert_any_call(api_key=self.from_api_key, base_url=self.from_instance)
        mock_lab_manager.assert_any_call(api_key=self.to_api_key, base_url=self.to_instance)

    @patch('pygeai.core.utils.console.Console.write_stdout')
    @patch('pygeai.migration.strategies.AILabManager')
    def test_artifact_type_migration_success(self, mock_lab_manager, mock_stdout):
        mock_source = Mock()
        mock_destination = Mock()
        mock_lab_manager.side_effect = [mock_source, mock_destination]

        mock_artifact_type = Mock(spec=ArtifactTypeDefinition)
        mock_artifact_type.name = "Test Artifact Type"
        mock_artifact_type.description = "Test Description"
        mock_artifact_type.tag = "test_tag"
        mock_source.get_artifact_type.return_value = mock_artifact_type
        mock_destination.create_artifact_type.return_value = mock_artifact_type

        strategy = ArtifactTypeMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            artifact_type_id=self.artifact_type_id
        )
        strategy.migrate()

        mock_source.get_artifact_type.assert_called_with(id_or_name=self.artifact_type_id)
        mock_destination.create_artifact_type.assert_called_with(
            name=mock_artifact_type.name,
            description=mock_artifact_type.description,
            tag=mock_artifact_type.tag
        )
        mock_stdout.assert_called_once()
        self.assertIn("New artifact_type detail:", mock_stdout.call_args[0][0])

    @patch('pygeai.migration.strategies.AILabManager')
    def test_artifact_type_migration_error_not_artifact_type(self, mock_lab_manager):
        mock_source = Mock()
        mock_destination = Mock()
        mock_lab_manager.side_effect = [mock_source, mock_destination]

        mock_source.get_artifact_type.return_value = Mock(spec=[])

        strategy = ArtifactTypeMigrationStrategy(
            from_api_key=self.from_api_key,
            from_instance=self.from_instance,
            to_api_key=self.to_api_key,
            to_instance=self.to_instance,
            artifact_type_id=self.artifact_type_id
        )

        with self.assertRaises(ValueError) as context:
            strategy.migrate()
        self.assertIn("Artifact_type migration failed", str(context.exception))
        mock_destination.create_artifact_type.assert_not_called()


if __name__ == '__main__':
    unittest.main()



from dataclasses import dataclass
from typing import Optional, Dict, Any, List
import uuid
import requests
from urllib3.exceptions import MaxRetryError
from pygeai.proxy.tool import ProxiedTool
from pygeai.core.utils.validators import validate_status_code
from pygeai.core.utils.parsers import parse_json_response
from pygeai.core.common.exceptions import InvalidAPIResponseException
from pygeai.core.utils.console import Console

@dataclass
class ToolProxyData:
    """
    Data class representing a tool proxy registration.

    :param id: Optional[uuid.UUID] - Unique identifier for the proxy. Defaults to None.
    :param name: Optional[str] - Name of the proxy. Defaults to None.
    :param description: Optional[str] - Description of the proxy. Defaults to None.
    :param affinity: Optional[uuid.UUID] - Affinity UUID. Defaults to None.
    :param tools: Optional[List[Tool]] - List of tools. Defaults to None.
    """
    id: Optional[uuid.UUID] = None
    name: Optional[str] = None
    description: Optional[str] = None
    affinity: Optional[uuid.UUID] = None
    tools: Optional[List[ProxiedTool]] = None

    def to_dict(self) -> Dict[str, Any]:
        """
        Convert the data to a dictionary for API requests.

        :return: Dict[str, Any] - Dictionary containing proxy data
        """
        return {
            "id": str(self.id) if self.id else None,
            "name": self.name,
            "description": self.description,
            "affinity": str(self.affinity) if self.affinity else str(uuid.UUID(int=0)),
            "tools": [
                {
                    "name": tool.get_full_name(),
                    **({"publicName": tool.get_public_name()} if tool.is_public() else {}),
                    "server": tool.server_name,
                    "description": tool.description,
                    "inputSchema": tool.format_for_llm()
                } for tool in self.tools
            ] if self.tools else []
        }

@dataclass
class ToolProxyJob:
    """
    Represents a Tool Proxy Job.

    :param id: uuid.UUID - Unique identifier for the job
    :param proxy_id: uuid.UUID - ID of the proxy handling the job
    :param proxy_status: str - Status of the proxy
    :param job_status: str - Status of the job
    :param input: Optional[str] - Input data for the job. Defaults to None.
    :param output: Optional[str] - Output data from the job. Defaults to None.
    :param server: Optional[str] - Server handling the job. Defaults to None.
    """
    id: uuid.UUID
    proxy_id: uuid.UUID
    proxy_status: str
    job_status: str
    input: Optional[str] = None
    output: Optional[str] = None
    server: Optional[str] = None

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'ToolProxyJob':
        """
        Create a ToolProxyJob from a dictionary.

        :param data: Dict[str, Any] - Dictionary containing job data
        :return: ToolProxyJob - New ToolProxyJob instance
        """
        return ToolProxyJob(
            id=uuid.UUID(data['id']),
            proxy_id=uuid.UUID(data['proxyId']),
            proxy_status=data['proxyStatus'],
            job_status=data['jobStatus'],
            input=data.get('input'),
            output=data.get('output'),
            server=data.get('server')
        )

@dataclass
class ToolProxyJobResult:
    """
    Represents the result of a Tool Proxy Job.

    :param success: bool - Whether the job was successful
    :param job: ToolProxyJob - The job that was executed
    """
    success: bool
    job: ToolProxyJob

    def to_dict(self) -> Dict[str, Any]:
        """
        Convert the result to a dictionary for API requests.

        :return: Dict[str, Any] - Dictionary containing job result
        """
        return {
            "success": self.success,
            "job": {
                "id": str(self.job.id),
                "jobStatus": self.job.job_status,
                "output": self.job.output
            }
        }


class ProxyClient:
    """
    Client for interacting with the GEAI API proxy.

    :param base_url: str - Base URL of the GEAI API
    :param api_key: str - API key for authentication
    :param proxy_id: uuid.UUID - ID of the proxy
    """

    def __init__(self, api_key: str, base_url: str, proxy_id: uuid.UUID ):
        """
        Initialize a new proxy client.

        :param base_url: str - Base URL of the GEAI API
        :param api_key: str - API key for authentication
        :param proxy_id: uuid.UUID - ID of the proxy
        """
        self.proxy_id = proxy_id
        self.base_url =  base_url.rstrip('/') + '/v2/tool-proxy'
        self.session = requests.Session()
        self.session.headers.update({'Authorization': f'Bearer {api_key}'})
        # Configurar el timeout
        self.session.timeout = (5, 30)  # (connect timeout, read timeout)

    def _make_request(self, method: str, endpoint: str, **kwargs) -> Dict[str, Any]:
        """
        Make a request to the proxy service.
        
        :param method: str - HTTP method (GET, POST, etc.)
        :param endpoint: str - API endpoint
        :param **kwargs: Additional arguments for requests
        :return: Dict[str, Any] - Response data as dictionary
        :raises: RequestException - For any request-related error
        """
        url = self.base_url + endpoint
        try:
            response = self.session.request(method, url, **kwargs)
            response.raise_for_status()
            try:
                validate_status_code(response)
                if not response.text or not response.text.strip():
                    return {}
                return parse_json_response(response, "unknown operation")
            except ValueError:
                return response.text
        except requests.exceptions.Timeout:
            raise TimeoutError(f"Request timed out for {url}")
        except requests.exceptions.HTTPError as e:
            raise requests.exceptions.HTTPError(f"HTTP error for {url}: {e}") from e
        except requests.exceptions.SSLError as e:
            raise requests.exceptions.SSLError(f"SSL error for {url}: {e}") from e
        except requests.exceptions.ProxyError as e:
            raise requests.exceptions.ProxyError(f"Proxy error for {url}: {e}") from e
        except requests.exceptions.ConnectionError as e:
            raise ConnectionError(f"Failed to connect to {url}") from e
        except MaxRetryError:
            raise
        except requests.exceptions.RequestException as e:
            raise requests.exceptions.RequestException(f"Request failed for {url}: {e}") from e
        except ValueError:
            return {"response": response.text}

    def register(self, proxy_data: ToolProxyData) -> Dict[str, Any]:
        """
        Register a new tool proxy.
        
        :param proxy_data: ToolProxyData - Object containing the proxy configuration and metadata
        :return: Dict[str, Any] - Dictionary with the registration response
        :raises: ConnectionError - If connection fails
        :raises: MaxRetryError - If max retries are exceeded
        :raises: requests.exceptions.RequestException - For other request errors
        """
        proxy_data.id = self.proxy_id
        data = {
            "server": proxy_data.to_dict()
        }
        return self._make_request('POST', '/register', json=data)
    
    def dequeue(self) -> List[ToolProxyJob]:
        """
        Dequeue a job from the proxy.

        :return: List[ToolProxyJob] - List of ToolProxyJob instances
        :raises: RequestException - For any request-related error
        """
        endpoint = f'/dequeue/{self.proxy_id}'
        response_data = self._make_request('GET', endpoint)
        return [ToolProxyJob.from_dict(job) for job in response_data]

    
    def send_result(self, result: ToolProxyJobResult):
        """
        Send the result of a tool proxy job.

        :param result: ToolProxyJobResult - Result object containing the job outcome
        :return: None
        :raises: RequestException - For any request-related error
        """
        endpoint = f'/result/{self.proxy_id}'
        data = {
            "jobResult": result.to_dict()
        }
        try:
            self._make_request('POST', endpoint, json=data)
        except InvalidAPIResponseException as e:
            Console.write_exception(f"Error sending result for job {self.proxy_id}:", e)


import unittest
from unittest.mock import patch, MagicMock
from json import JSONDecodeError
from pygeai.assistant.rag.clients import RAGAssistantClient
from pygeai.core.common.exceptions import APIResponseError


class TestRAGAssistantClient(unittest.TestCase):
    """
    python -m unittest pygeai.tests.assistants.rag.test_clients.TestRAGAssistantClient
    """
    def setUp(self):
        with patch('pygeai.core.base.clients.BaseClient.__init__', return_value=None):
            self.client = RAGAssistantClient()
        self.mock_response = MagicMock()
        self.client.api_service = MagicMock()
        self.assistant_name = "test_assistant"
        self.document_id = "doc_123"
        self.file_path = "/path/to/test.pdf"

    def test_get_url_safe_name(self):
        name = "test assistant"
        expected_safe_name = "test+assistant"

        result = self.client.get_url_safe_name(name)

        self.assertEqual(result, expected_safe_name)

    def test_get_assistants_from_project_successful_json_response(self):
        expected_response = {"assistants": []}
        self.mock_response.json.return_value = expected_response
        self.mock_response.status_code = 200
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.get_assistants_from_project()

        self.client.api_service.get.assert_called_once()
        self.assertEqual(result, expected_response)

    def test_get_assistants_from_project_json_decode_error(self):
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.mock_response.text = "Invalid JSON response"
        self.mock_response.status_code = 500
        self.client.api_service.get.return_value = self.mock_response

        with self.assertRaises(APIResponseError) as context:
            self.client.get_assistants_from_project()

        self.client.api_service.get.assert_called_once()
        self.assertIn("API returned an error", str(context.exception))  # "Unable to get assistants from project", str(context.exception))

    def test_get_assistant_data_successful_json_response(self):
        expected_response = {"name": self.assistant_name, "details": "test"}
        self.mock_response.json.return_value = expected_response
        self.mock_response.status_code = 200
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.get_assistant_data(self.assistant_name)

        self.client.api_service.get.assert_called_once()
        self.assertEqual(result, expected_response)

    def test_get_assistant_data_json_decode_error(self):
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.mock_response.text = "Invalid JSON response"
        self.mock_response.status_code = 500
        self.client.api_service.get.return_value = self.mock_response

        with self.assertRaises(APIResponseError) as context:
            self.client.get_assistant_data(self.assistant_name)

        self.client.api_service.get.assert_called_once()
        self.assertIn("API returned an error", str(context.exception))  # "Unable to get assistant data", str(context.exception))

    def test_create_assistant_successful_json_response(self):
        name = "new_assistant"
        description = "Test description"
        expected_response = {"id": "new_id", "name": name}
        self.mock_response.json.return_value = expected_response
        self.mock_response.status_code = 200
        self.client.api_service.post.return_value = self.mock_response

        result = self.client.create_assistant(name, description)

        self.client.api_service.post.assert_called_once()
        self.assertEqual(result, expected_response)

    def test_create_assistant_json_decode_error(self):
        name = "new_assistant"
        description = "Test description"
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.mock_response.text = "Error in response"
        self.mock_response.status_code = 500
        self.client.api_service.post.return_value = self.mock_response

        with self.assertRaises(APIResponseError) as context:
            self.client.create_assistant(name, description)

        self.client.api_service.post.assert_called_once()
        self.assertIn("API returned an error", str(context.exception))  # "Unable to create assistant", str(context.exception))

    def test_update_assistant_successful_json_response(self):
        name = "test_assistant"
        status = 1
        expected_response = {"name": name, "status": status}
        self.mock_response.json.return_value = expected_response
        self.mock_response.status_code = 200
        self.client.api_service.put.return_value = self.mock_response

        result = self.client.update_assistant(name, status)

        self.client.api_service.put.assert_called_once()
        self.assertEqual(result, expected_response)

    def test_update_assistant_json_decode_error(self):
        name = "test_assistant"
        status = 2
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.mock_response.text = "Update failed"
        self.mock_response.status_code = 500
        self.client.api_service.put.return_value = self.mock_response

        with self.assertRaises(APIResponseError) as context:
            self.client.update_assistant(name, status)

        self.client.api_service.put.assert_called_once()
        self.assertIn("API returned an error", str(context.exception))  # "Unable to update assistant", str(context.exception))

    def test_delete_assistant_successful_json_response(self):
        name = "test_assistant"
        expected_response = {"message": "Assistant deleted"}
        self.mock_response.json.return_value = expected_response
        self.mock_response.status_code = 200
        self.client.api_service.delete.return_value = self.mock_response

        result = self.client.delete_assistant(name)

        self.client.api_service.delete.assert_called_once()
        self.assertEqual(result, expected_response)

    def test_delete_assistant_json_decode_error(self):
        name = "test_assistant"
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.mock_response.text = "Delete error"
        self.mock_response.status_code = 500
        self.client.api_service.delete.return_value = self.mock_response

        with self.assertRaises(APIResponseError) as context:
            self.client.delete_assistant(name)

        self.client.api_service.delete.assert_called_once()
        self.assertIn("API returned an error", str(context.exception))  # "Unable to delete assistant", str(context.exception))

    def test_get_documents_successful_json_response(self):
        name = "test_assistant"
        skip = 0
        count = 10
        expected_response = {"documents": []}
        self.mock_response.json.return_value = expected_response
        self.mock_response.status_code = 200
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.get_documents(name, skip, count)

        self.client.api_service.get.assert_called_once()
        self.assertEqual(result, expected_response)

    def test_get_documents_json_decode_error(self):
        name = "test_assistant"
        skip = 0
        count = 10
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.mock_response.text = "Documents error"
        self.mock_response.status_code = 500
        self.client.api_service.get.return_value = self.mock_response

        with self.assertRaises(APIResponseError) as context:
            self.client.get_documents(name, skip, count)

        self.client.api_service.get.assert_called_once()
        self.assertIn("API returned an error", str(context.exception))  # "Unable to get documents", str(context.exception))

    def test_delete_all_documents_successful_json_response(self):
        name = "test_assistant"
        expected_response = {"message": "All documents deleted"}
        self.mock_response.json.return_value = expected_response
        self.mock_response.status_code = 200
        self.client.api_service.delete.return_value = self.mock_response

        result = self.client.delete_all_documents(name)

        self.client.api_service.delete.assert_called_once()
        self.assertEqual(result, expected_response)

    def test_delete_all_documents_json_decode_error(self):
        name = "test_assistant"
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.mock_response.text = "Delete all error"
        self.mock_response.status_code = 500
        self.client.api_service.delete.return_value = self.mock_response

        with self.assertRaises(APIResponseError) as context:
            self.client.delete_all_documents(name)

        self.client.api_service.delete.assert_called_once()
        self.assertIn("API returned an error", str(context.exception))  # "Unable to delete all documents", str(context.exception))

    def test_retrieve_document_successful_json_response(self):
        name = "test_assistant"
        document_id = "doc_123"
        expected_response = {"id": document_id, "content": "test content"}
        self.mock_response.json.return_value = expected_response
        self.mock_response.status_code = 200
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.retrieve_document(name, document_id)

        self.client.api_service.get.assert_called_once()
        self.assertEqual(result, expected_response)

    def test_retrieve_document_json_decode_error(self):
        name = "test_assistant"
        document_id = "doc_123"
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.mock_response.text = "Retrieve error"
        self.mock_response.status_code = 500
        self.client.api_service.get.return_value = self.mock_response

        with self.assertRaises(APIResponseError) as context:
            self.client.retrieve_document(name, document_id)

        self.client.api_service.get.assert_called_once()
        self.assertIn("API returned an error", str(context.exception))  # "Unable to retrieve document", str(context.exception))

    def test_upload_document_binary_successful_json_response(self):
        name = "test_assistant"
        file_path = "/path/to/test.pdf"
        upload_type = "binary"
        content_type = "application/pdf"
        expected_response = {"id": "uploaded_doc", "status": "success"}
        self.mock_response.json.return_value = expected_response
        self.mock_response.status_code = 200
        self.client.api_service.post_file_binary.return_value = self.mock_response

        with patch('builtins.open', return_value=MagicMock()):
            result = self.client.upload_document(name, file_path, upload_type, content_type=content_type)

        self.client.api_service.post_file_binary.assert_called_once()
        self.assertEqual(result, expected_response)

    def test_upload_document_multipart_successful_json_response(self):
        name = "test_assistant"
        file_path = "/path/to/test.pdf"
        upload_type = "multipart"
        content_type = "application/pdf"
        metadata = {"key": "value"}
        expected_response = {"id": "uploaded_doc", "status": "success"}
        self.mock_response.json.return_value = expected_response
        self.mock_response.status_code = 200
        self.client.api_service.post_files_multipart.return_value = self.mock_response

        with patch('builtins.open', return_value=MagicMock()), \
             patch('pathlib.Path.name', return_value="test.pdf"):
            result = self.client.upload_document(name, file_path, upload_type, metadata, content_type)

        self.client.api_service.post_files_multipart.assert_called_once()
        self.assertEqual(result, expected_response)

    def test_upload_document_invalid_upload_type(self):
        name = "test_assistant"
        file_path = "/path/to/test.pdf"
        upload_type = "invalid"

        with self.assertRaises(ValueError) as context:
            self.client.upload_document(name, file_path, upload_type)

        self.assertEqual(str(context.exception), "Invalid upload_type. Use 'binary' or 'multipart'.")

    def test_upload_document_json_decode_error(self):
        name = "test_assistant"
        file_path = "/path/to/test.pdf"
        upload_type = "binary"
        content_type = "application/pdf"
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.mock_response.text = "Upload error"
        self.mock_response.status_code = 500
        self.client.api_service.post_file_binary.return_value = self.mock_response

        with patch('builtins.open', return_value=MagicMock()):
            with self.assertRaises(APIResponseError) as context:
                self.client.upload_document(name, file_path, upload_type, content_type=content_type)

        self.client.api_service.post_file_binary.assert_called_once()
        self.assertIn("API returned an error", str(context.exception))  # "Unable to upload document", str(context.exception))

    def test_delete_document_successful_json_response(self):
        name = "test_assistant"
        document_id = "doc_123"
        expected_response = {"message": "Document deleted"}
        self.mock_response.json.return_value = expected_response
        self.mock_response.status_code = 200
        self.client.api_service.delete.return_value = self.mock_response

        result = self.client.delete_document(name, document_id)

        self.client.api_service.delete.assert_called_once()
        self.assertEqual(result, expected_response)

    def test_delete_document_json_decode_error(self):
        name = "test_assistant"
        document_id = "doc_123"
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.mock_response.text = "Delete document error"
        self.mock_response.status_code = 500
        self.client.api_service.delete.return_value = self.mock_response

        with self.assertRaises(APIResponseError) as context:
            self.client.delete_document(name, document_id)

        self.client.api_service.delete.assert_called_once()
        self.assertIn("API returned an error", str(context.exception))  # "Unable to delete document", str(context.exception))

    def test_execute_query_successful_json_response(self):
        query = {"text": "test query"}
        expected_response = {"result": "query response"}
        self.mock_response.json.return_value = expected_response
        self.mock_response.status_code = 200
        self.client.api_service.post.return_value = self.mock_response

        result = self.client.execute_query(query)

        self.client.api_service.post.assert_called_once()
        self.assertEqual(result, expected_response)

    def test_execute_query_json_decode_error(self):
        query = {"text": "test query"}
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.mock_response.text = "Query error"
        self.mock_response.status_code = 500
        self.client.api_service.post.return_value = self.mock_response

        with self.assertRaises(APIResponseError) as context:
            self.client.execute_query(query)

        self.client.api_service.post.assert_called_once()
        self.assertIn("API returned an error", str(context.exception))  # "Unable to execute query", str(context.exception))
